
# -*- coding: utf-8 -*-
"""
Created on 

@author: OWP
"""

#%%

import numpy as np
import putools
import warnings

def warning_on_one_line(message, category, filename, lineno, file=None, line=None):
    return '%s:%s: %s: %s\n' % (filename, lineno, category.__name__, message)

warnings.formatwarning = warning_on_one_line

#%%

def assembly(fid,assembly_name):
    
    '''
    *ASSEMBLY
    
    Arguments
    ------------
    fid: file identifier
    assembly_name: string with name
    
    Returns
    ------------
    None
    
    '''

    fid.write('*ASSEMBLY, NAME=' + assembly_name.upper() + '\n')
    fid.write('**' + '\n')
    
#%%

def assemblyend(fid):

    '''
    *END ASSEMBLY
    
    Arguments
    ------------
    fid: file identifier
    
    Returns
    ------------
    None
    
    '''

    fid.write('*END ASSEMBLY' + '\n')
    fid.write('**' + '\n')
    
#%%

def beamaddedinertia(fid,linear_mass,x1,x2,alpha,I_11,I_22,I_12):

    '''
    *BEAM ADDED INERTIA
    
    Arguments
    ------------
    fid: file identifier
    linear_mass: mass [kg/m]
    x1: 1-dir offset [m]
    x2: 2-dir offset [m]
    alpha: rotation offset [deg]
    I_11: inertia [kg*m^2]
    I_22: inertia [kg*m^2]
    I_12: inertia [kg*m^2]
    
    Returns
    ------------
    None
    
    '''

    fid.write('*BEAM ADDED INERTIA' + '\n')
    
    values_list=[linear_mass,x1,x2,alpha,I_11,I_22,I_12]
    str_values=putools.num.num2stre(values_list,digits=3,delimeter=', ')
    fid.write(str_values + '\n')

    #fid.write('**' + '\n')
    
#%%

def beamsection(fid,elset,material,sectiontype,sectionproperties,direction):
    
    '''
    *BEAM SECTION
    
    Arguments
    ------------
    fid: file identifier
    elset: string with element set
    material: string with material name
    sectiontype: e.g. 'RECTANGULAR' or 'PIPE'
    sectionproperties: array with numbers required for the section type above
    direction: array with n1-direction, e.g. [0,1,0]
    
    Returns
    ------------
    None
    
    '''
    
    if isinstance(elset,str):
        elset=[elset]

    sectionproperties=putools.num.ensurenp(sectionproperties)
    direction=putools.num.ensurenp(direction)

    for elset_sub in elset:
        fid.write('*BEAM SECTION, ELSET=' + elset_sub.upper() + ', MATERIAL=' + material.upper() + ', SECTION=' + sectiontype.upper() + '\n')

        putools.txt.writematrix(fid,sectionproperties,5,', ','f')
        
        putools.txt.writematrix(fid,direction,3,', ','f')
    
    fid.write('**' + '\n')
    
    
#%%

def beamgeneralsection(fid,elset,density,sectionproperties,direction,materialproperties):
    
    '''
    *BEAM GENERAL SECTION
    
    Arguments
    ------------
    fid: file identifier
    elset: string with element set name
    density: in [kg/m^3]
    sectionproperties: [A,I11,I12,I22,It] array with cross section properties
    direction: array with n1-direction, e.g. [0,1,0]
    materialproperties: [E,G] array with elastic and shear modulus
    
    Returns
    ------------
    None
    
    '''
    
    fid.write('*BEAM GENERAL SECTION, ELSET=' + elset.upper() + ', SECTION=GENERAL, DENSITY=' + putools.num.num2strf(density,3) + '\n')
    
    putools.txt.writematrix(fid,sectionproperties,5,', ','e')
    
    putools.txt.writematrix(fid,direction,3,', ','f')
    
    putools.txt.writematrix(fid,materialproperties,5,', ','e')
    
    fid.write('**' + '\n')
    
#%%

def boundary(fid,op,nodename,BCmat,partname):

    '''
    *BOUNDARY
    
    Arguments
    ------------
    fid: file identifier
    op: 'MOD' for modified BCs or 'NEW' for new BCs (erase all old) 
    nodename: string with node set name or array with node numbers
    BCmat: array to specificy, e.g. [1,4,0] to set DOF 1,2,3,4 to zero
    partname: string with part name
    
    Returns
    ------------
    None
    
    '''
    
    checkarg(op,['MOD','NEW'])
    
    partname_str=''
    if len(partname)>0:
        partname_str=partname.upper() + '.'
        
    fid.write('*BOUNDARY, OP=' + op.upper() + '\n')
    
    if isinstance(nodename,str):
        nodename=[nodename]
        
    if putools.num.isnumeric(nodename):
        nodename=putools.num.ensurenp(nodename)
        for k in nodename:
            fid.write(partname_str + str(nodename[k]) + ',' + str(BCmat[0]) + ',' + str(BCmat[1]) + ',' + str(BCmat[2]) + '\n')
    elif isinstance(nodename,list):
        for node in nodename:
            fid.write(partname_str + str(node) + ',' + str(BCmat[0]) + ',' + str(BCmat[1]) + ',' + str(BCmat[2]) + '\n')
  
    fid.write('**' + '\n')
    fid.write('**' + '\n')
    
#%%

def checkarg(a_str,arg_allowed):

    '''
    Check allowable arguments
    
    Arguments
    ------------
    a_str: string with argument to check
    arg_allowed: string or list with arguments that are allowed
    
    Returns
    ------------
    None
    
    '''
    
    if isinstance(arg_allowed,str):
        arg_allowed=[arg_allowed]
    
    CorrectArg=False
    for arg in arg_allowed:
        if a_str.upper()==arg.upper():
            CorrectArg=True

    if CorrectArg==False:
        
        exc_str='Argument ' + a_str + ' not allowed, argument must be '
        for arg in arg_allowed:
            exc_str=exc_str + arg.upper() + ' or '
        
        exc_str=exc_str[:-4]
        raise Exception(exc_str)
        
#%%
def cload(fid,op,nset,dof,magnitude_force,partname=''):

    '''
    *CLOAD
    
    Arguments
    ------------
    fid: file identifier
    op: 'MOD' for modified CLOADS or 'NEW' for new CLOADS (erase all old) 
    nset: string with node set name or array with node numbers
    dof: DOF number between 1 and 6
    magnitude_force: signed force magnitude
    
    Returns
    ------------
    None
    
    '''
    
    checkarg(op,['MOD','NEW','DELETE'])

    partname_str=''
    if len(partname)>0:
        partname_str=partname.upper() + '.'

    if op.casefold()=='DELETE':
        fid.write('*CLOAD, OP=NEW' + '\n')
        fid.write('**' + '\n')
        return
    
    fid.write('*CLOAD, OP=' + op.upper() + '\n')
    
    magnitude_force=putools.num.ensurenp(magnitude_force)
    
    if isinstance(nset,str):
        nset=[nset]
    
    magnitude_force=np.atleast_1d(magnitude_force)
    
    if np.shape(magnitude_force)==(1,):
        magnitude_force=magnitude_force*np.ones(len(nset))
    
    if putools.num.isnumeric(nset):
        for k in np.arange(len(nset)):
            fid.write( partname_str + str(nset[k]) + ', ' + str(int(dof)) + ', ' + putools.num.num2stre(magnitude_force[k],3) + '\n')
    elif isinstance(nset,list):
        for k in np.arange(len(nset)):
            fid.write( partname_str + nset[k] + ', ' + str(int(dof)) + ', ' + putools.num.num2stre(magnitude_force[k],3) + '\n')
        
    fid.write('**' + '\n')
    #fid.write('**' + '\n')
    
#%%

def comment(fid,comment_str,logic_main=False):

    '''
    Add comment to input file
    
    Arguments
    ------------
    fid: file identifier
    comment_str: string or list with comments
    logic_main: many or few stars
    
    Returns
    ------------
    None
    
    '''

    if logic_main==True:
        separatator='***********************************************************'
    else:
        separatator='**********'

    if isinstance(comment_str,str):
        comment_str=[comment_str]

    fid.write(separatator + '\n')
    for comment_sub in comment_str:
        fid.write('** ' + comment_sub + '\n')

    fid.write(separatator + '\n')


#%%

def dload(fid,op,elset,type_id,magnitude):

    '''
    *DLOAD

    Arguments
    ------------
    fid: file identifier
    op: 'MOD' for modified DLOADS or 'NEW' for new DLOADS (erase all old) 
    elset: name of element set
    type_id: load type, e.g. PZ for line load in z-direction
    magnitude: signed load magnitude
    
    Returns
    ------------
    None
    
    '''
    
    checkarg(op,['MOD','NEW','DELETE'])

    if op.upper()=='DELETE':
        fid.write('*DLOAD, OP=NEW' + '\n')
        fid.write('**' + '\n')
        fid.write('**' + '\n')
        return
        
    if isinstance(magnitude,str):
        magnitude_str=magnitude
    elif putools.num.isnumeric(magnitude):
        magnitude_str=putools.num.num2stre(magnitude)
        
    fid.write('*DLOAD, OP=' + op.upper() + '\n')
    fid.write( elset + ', ' + type_id + ', ' + magnitude_str + '\n')
    
    fid.write('**' + '\n')
    #fid.write('**' + '\n')
    
#%%
    
def element(fid,element_nodenumber,element_type,elsetname,star=True):
    
    '''
    *ELEMENT
    
    Arguments
    ------------
    fid: file identifier
    element_nodenumber: array with rows [el_num,node_num1,node_num2]
    element_type: element type, e.g. B31
    elsetname: string with name
    
    Returns
    ------------
    None
    
    '''
    
    element_nodenumber=putools.num.ensurenp(element_nodenumber)
    element_nodenumber=np.atleast_2d(element_nodenumber)

    id_neg=element_nodenumber<=0
    n_neg=np.sum(np.sum(id_neg))
    if n_neg>0:
        print('***** For ELSET ' + elsetname)
        raise Exception('***** Negative element or node number' )
    

    if element_type=='B31' or element_type=='B33': 
        if np.shape(element_nodenumber)[1]!=3:
            print('***** For ELSET ' + elsetname)
            raise Exception('***** B31 or B33 must have 2 nodes')
        
    if element_type=='B32':
        if np.shape(element_nodenumber)[1]!=4:
            print('***** For ELSET ' + elsetname)
            raise Exception('***** B32 must have 3 nodes')
            
    fid.write('*ELEMENT, TYPE=' + element_type.upper() + ', ELSET=' + elsetname.upper() + '\n')
    
    putools.txt.writematrix(fid,element_nodenumber,'',', ','int')
    
    if star==True:
        fid.write('**' + '\n')
        fid.write('**' + '\n')


#%%

def memberjointc(fid,node1,node2,coord1,coord2,node_num_base,el_num_base,element_type,setname,n1,kj1,kj2,offset1=0,offset2=0,n_el=10,max_length=None):
                
    '''
    Beam member with user-defined stiffness at member joints
    
    N1 J1     MemberEl1    MemberEl2    MemberEl3    MemberEl4    MemberEl5     J2 N2
    O~~~~~~O------------o-------------o------------o------------o------------O~~~~~~O

    Arguments
    ------------
    fid: file identifier
    node1: (super)node number at joint 1
    node2: (super)node number at joint 2
    coord1: coordinates of node 1
    coord2: coordinates of node 2
    node_num_base: base node number for member
    el_num_base: base element number for member
    element_type: element type, e.g. B31
    setname: name for member
    n1: array with n1-direction, e.g. [0,1,0]
    kj1: [kx,ky,kz,krx,kry,krz] array with 6 spring stiffnesses in [N/m] and [Nm/rad] for joint 1
    kj2: [kx,ky,kz,krx,kry,krz] array with 6 spring stiffnesses in [N/m] and [Nm/rad] for joint 2
    offset1: eccentricity offset of member end 1 in [m]
    offset2: eccentricity offset of member end 2 in [m]
    n_el: number of elements in member
    max_length: max element length (overrides n_el)

    Returns
    ------------
    None
    
    '''
    
    setname=setname.upper()
    
    comment(fid,'Member ' + setname)
    
    if el_num_base is None:
        el_num_base=node_num_base
    
    # Joint stiffness
    kj1=putools.num.ensurenp(kj1)
    kj2=putools.num.ensurenp(kj2)
    
    # Only allow 2-node elements
    checkarg(element_type,['B31','B33'])
        
    if any(kj1<0):
        raise Exception('***** kj1 is negative for set ' + setname)
    
    if any(kj2<0):
        raise Exception('***** kj2 is negative for set ' + setname)
    
    if offset1<0:
        raise Exception('***** offset1 is negative for set ' + setname)
    
    if offset2<0:
        raise Exception('***** offset2 is negative for set ' + setname)
        
    if all(kj1==0):
        warnings.warn('***** kj1 is all zero for set ' + setname, stacklevel=2)
    
    if all(kj2==0):
        warnings.warn('***** kj2 is all zero for set ' + setname, stacklevel=2)

    # if any(kj1>1e36) and not all(kj1>1e36):
        # raise Exception('***** kj1 is too large for set ' + setname)
    
    # if any(kj2>1e36) and not all(kj2>1e36):
        # raise Exception('***** kj2 is too large for set ' + setname)

    # Vector for n1-direction (lateral)
    n1=putools.num.ensurenp(n1)
    n1=n1/np.linalg.norm(n1)
    
    coord1=coord1.flatten()
    coord2=coord2.flatten()
    
    # Vector along member
    t_vec=putools.num.ensurenp(coord2)-putools.num.ensurenp(coord1)
    L0=np.linalg.norm(t_vec)
    
    t_vec=t_vec/L0
    
    n1_tmp=n1
    n2=np.cross(t_vec,n1_tmp)
    n1=np.cross(n2,t_vec)

    if all(np.abs(t_vec)<1e-12):
        raise Exception('***** t_vec is zero for set ' + setname)
        
    if all(np.abs(n1)<1e-12):
        raise Exception('***** n1 is zero for set ' + setname)
        
    if all(np.abs(n2)<1e-12):
        raise Exception('***** n2 is zero for set ' + setname)
        
    if L0==0:
        raise Exception('***** L0 is zero for set ' + setname)
    
    # If kj stiffnesses are all inf, the member is continuous and therefore
    # instead directly linked to the supernode, the offset is thus ignored
        
    # In the future, this should be replaced by an input argument instead
        
    inf_threshold=1e36
        
    if all(kj1>inf_threshold):
        J1_cont=True
        offset1=0
    else:
        J1_cont=False
        
    if all(kj2>inf_threshold):
        J2_cont=True
        offset2=0
    else:
        J2_cont=False     
        
    # Shorten member by offset
    if offset1>0:
        coord1=coord1+t_vec*offset1
        
    if offset2>0:
        coord2=coord2-t_vec*offset2
        
    # Determine number of elements
    if (n_el is None):
        n_el=np.ceil(L0/max_length) 
    elif max_length is not None:
    
        # If n_el is specified then use it as a miniumum, else use only length
        if n_el is not None:
            n_el=np.max([n_el,np.ceil(L0/max_length)])
        elif n_el is None:
            n_el=np.ceil(L0/max_length)
            
    n_el=int(n_el)
    
    # Member nodes and elements
    x=np.linspace(coord1[0],coord2[0],n_el+1)    
    y=np.linspace(coord1[1],coord2[1],n_el+1)    
    z=np.linspace(coord1[2],coord2[2],n_el+1)
    
    node_num=np.arange(1,len(x)+1)+node_num_base
    el_num=np.arange(1,len(x))+el_num_base
    
    # Write nodes and elements
    node(fid,np.column_stack((node_num,x,y,z)),setname)
    
    el_matrix=np.column_stack((el_num,node_num[0:-1],node_num[1:]))
    
    if J1_cont==True:
        el_matrix[0,1]=node1
    
    if J2_cont==True:
        el_matrix[-1,-1]=node2
    
    # Write elements of member
    element(fid,el_matrix,element_type,setname)

    # Create local orientation system along member
    # a=node2
    # b=node_num[0]-1
    # c=node1
    
    #fid.write('** Extra node (b) for definition of orientation system' + '\n')
    #node(fid,np.array([b,x[0]+n1[0],y[0]+n1[1],z[0]+n1[2]]),setname + '_BNODE')
    #orientation(fid,setname + '_LOCSYS','NODES',[a,b,c])
 
    fid.write('** Local coordinate system' + '\n')
    fid.write('** t: ')
    putools.txt.writematrix(fid,t_vec,format='f')
    fid.write('** n1: ')
    putools.txt.writematrix(fid,n1,format='f')
    fid.write('** n2: ')
    putools.txt.writematrix(fid,n2,format='f')
    
    coord_b=np.array([x[0]+n1[0],y[0]+n1[1],z[0]+n1[2]])
    data=np.hstack((coord2,coord_b,coord1))
    orientation(fid,setname + '_LOCSYS','coordinates',data)
    
    DOFS=['1','2','3','4','5','6']
    
    if all(kj1[0:2]>inf_threshold) and any(kj1[3:]<inf_threshold):
        J1_link=True
    else:
        J1_link=False
        
    if all(kj2[0:2]>inf_threshold) and any(kj2[3:]<inf_threshold):
        J2_link=True
    else:
        J2_link=False
        
        
    #Turn off for now
    J1_link=False
    J2_link=False
    
    # Joints
    for j in np.arange(0,2):
        
        if j==0:
            joint_str='J1'
            kj=kj1
            el_node_matrix=[el_num[-1]+1,node1,node_num[0]]
            
            if J1_cont==True:
                line(fid,'** Joint J1 for ' + setname + ' is continuous, no stiffness defined') #
                line(fid,'**')
                continue
            
        elif j==1:
            joint_str='J2'
            kj=kj2
            el_node_matrix=[el_num[-1]+2,node_num[-1],node2]
            
            if J2_cont==True:
                line(fid,'** Joint J2 for ' + setname + ' is continuous, no stiffness defined') #
                line(fid,'**')
                continue
              
        line(fid,'**')
        line(fid,'** Joint ' + joint_str)
        
        elset_name=setname + '_' + joint_str
        
        element(fid,el_node_matrix,'JOINTC',elset_name,star=False)
        
        ori_str=', ORIENTATION=' + setname + '_LOCSYS'
        
        line(fid,'*JOINT, ELSET=' + elset_name + ori_str) #
        
        for k in np.arange(0,6):
            
            if j==0 and J1_link==True and (k==0 or k==1 or k==2):
                fid.write('** MPC link created for DOF ' + DOFS[k] + '\n')
                continue
            
            if j==1 and J2_link==True and (k==0 or k==1 or k==2):
                fid.write('** MPC link created for DOF ' + DOFS[k] + '\n')
                continue
            
            if kj[k]==0:
                fid.write('** Zero joint stiffness in DOF ' + DOFS[k] + '\n')
                continue
                        
            # From Abaqus doc:
            # If the *SPRING option is being used to define part of the behavior 
            # of ITS or JOINTC elements, it must be used in conjunction with the
            # *ITS or *JOINT options and the ELSET and ORIENTATION parameters
            # should not be used.
            str_el=''
                        
            fid.write('*SPRING' + str_el + '\n')
            fid.write(DOFS[k] + ',' + DOFS[k] + '\n')
            putools.txt.writematrix(fid,kj[k],3,',','e')
    
        
        line(fid,'**')
        line(fid,'**')
        
        if j==0 and J1_link==True:
            mpc(fid,'LINK',[node_num[0],node1])
            
        if j==1 and J2_link==True:
            mpc(fid,'LINK',[node_num[-1],node2])
      
    fid.write('**' + '\n')
    fid.write('**' + '\n')    


#%%


def getcoord(nodes,node_matrix):
    
    nodes=putools.num.ensure_1d_list(nodes)

    coord=np.zeros((len(nodes),3))
    for k in np.arange(len(nodes)):
        
        idx=np.where(node_matrix[:,0]==int(nodes[k]))[0][0]
        coord[k,:]=node_matrix[idx,1:]
    
    return coord
    
def member_prop(list_el,k,num_base):

    el_base=list_el[k][0]+num_base
    node1=list_el[k][1]+num_base
    node2=list_el[k][2]+num_base
    
    k_type1=list_el[k][3]
    k_type2=list_el[k][4]    
    
    offset1=list_el[k][5]
    offset2=list_el[k][6]
    
    return el_base,node1,node2,k_type1,k_type2,offset1,offset2

def memberjointc_wrap(fid,node_matrix,members,sec_dict,kj_dict,setname,node_offset,max_length):
        
    '''
    Member with joints at ends (wrapper)
    
    Arguments
    ------------
    fid: file identifier
    node_matrix: array with rows [node_num,coord_x,coord_y,coord_z]
    members: dict
    sec_dict: dict
    kj_dict: list of dict
    setname: name of nodes and elements in member
    node_offset: offset of node and element numbers
    max_length: max length of elements in member

    Returns
    ------------
    None
    
    '''
    
    setname_all=[]
    
    for k in np.arange(len(members)):
          
        (el_base,node1,node2,k_type1,k_type2,offset1,offset2)=member_prop(members,k,node_offset)

        kj1,kj2=None,None

        for idx in np.arange(len(kj_dict)):
            
            if kj_dict[idx]['name']==k_type1:
                kj1=kj_dict[idx]['data']
          
            if kj_dict[idx]['name']==k_type2:
                kj2=kj_dict[idx]['data']
                
        if kj1 is None:
                raise Exception('***** Missing data for joint stiffness ' + k_type1)
                
        if kj2 is None:
                raise Exception('***** Missing data for joint stiffness ' + k_type2)
                
        coord1=getcoord(node1,node_matrix)
        coord2=getcoord(node2,node_matrix)
        
        elsetname=setname + '_' + str(int(el_base))
    
        memberjointc(fid,node1,node2,coord1,coord2,el_base*100,None,'B31',elsetname,sec_dict['dir'],kj1,kj2,offset1=offset1,offset2=offset2,max_length=max_length,n_el=None) 
          
        if sec_dict['type']=='general':
            beamgeneralsection(fid,elsetname,sec_dict['rho'],sec_dict['data'],sec_dict['dir'],[sec_dict['E'],sec_dict['G']])
        else:
            beamsection(fid,elsetname,sec_dict['material'],sec_dict['type'],sec_dict['data'],sec_dict['dir'])
              
        
        setname_all.append(elsetname)
        
        
        
    elset(fid,setname,setname_all)
    nset(fid,setname,setname_all)
    
#%%

def fieldoutput(fid,id_type,variables,set_id='',options=''):
    
    '''
    *FIELD OUTPUT
    
    Arguments
    ------------
    fid: file identifier
    id_type: 'NODE' or 'ELEMENT'
    variables: response quantity, e.g. U or SF
    set_id: name of nodeset or elset
    
    Returns
    ------------
    None
    
    '''

    comma=', '
    if len(options)<=1:
        comma=''
        options=''

    fid.write('*OUTPUT, FIELD' + comma + options.upper() + '\n')
    
    if id_type.upper()=='NODE':
        if not set_id:
            fid.write('*NODE OUTPUT \n')
        else:
            fid.write('*NODE OUTPUT, NSET=' + set_id.upper() + '\n')
        
    elif id_type.upper()=='ELEMENT':
        if not set_id:
            fid.write('*ELEMENT OUTPUT \n')
        else:
            fid.write('*ELEMENT OUTPUT, ELSET=' + set_id.upper() + '\n')
        
    if isinstance(variables,str):
        variables=[variables]
    
    for variables_sub in variables:
        fid.write(variables_sub + '\n')
    
    fid.write('**' + '\n')
    
#%%

def elset(fid,elsetname,elements,option=''):
    
    '''
    *ELSET
    
    Arguments
    ------------
    fid: file identifier
    elsetname: string with name
    elements: element numbers or list of strings to include in elset
    
    Returns
    ------------
    None
    
    '''
    
    fid.write('*ELSET, ELSET=' + elsetname.upper() + '\n')
    
    if isinstance(elements,str):
        
        elements=[elements]
        
    if putools.num.isnumeric(elements):
        elements=np.atleast_1d(putools.num.ensurenp(elements))
        bins=putools.num.rangebin(len(elements),16)
        for bins_sub in bins:
            putools.txt.writematrix(fid,elements[bins_sub],'',',','int')
    else:
        for elements_sub in elements:
            fid.write(elements_sub.upper() + '\n')
                    
    fid.write('**' + '\n')
    fid.write('**' + '\n')
        
#%%

def frequency(fid,n_modes,normalization='DISPLACEMENT'):
    
    '''
    *FREQUENCY
    
    Arguments
    ------------
    fid: file identifier
    n_modes: number of modes
    normalization: 'DISPLACEMENT' or 'MASS'
    
    Returns
    ------------
    None
    
    '''
    
    if normalization.upper()=='MASS':
        norm='MASS'
        comma=''
        options=''
    else:
        norm='DISPLACEMENT'
        comma=', '
        options='SIM=NO'

    fid.write('*FREQUENCY, NORMALIZATION=' + norm + comma + options + '\n')
    
    fid.write(str(n_modes) + '\n')
    
    fid.write('**' + '\n')
    fid.write('**' + '\n')
        
#%%

def gravload(fid,op,elset,magnitude=9.81,direction='z',partname=''):
    
    '''
    Gravity load through *DLOAD
    
    Arguments
    ------------
    fid: file identifier
    op: 'MOD' or 'NEW'
    elset: element set
    magnitude: gravitational constant
    direction: 'x','y', or 'z'
    partname: name of part
    
    Returns
    ------------
    None
    
    '''

    partname_str=''
    if len(partname)>0:
        partname_str=partname.upper() + '.'
                
    checkarg(op,['MOD','NEW'])
    
    checkarg(direction,['x','y','z'])
    
    fid.write('*DLOAD, OP=' + op.upper() + '\n')
    
    if isinstance(elset,str):
        elset=[elset]
    
    if magnitude<0:
        print('***** Gravity magnitude should generally be positive: magnitude 9.81 and direction [0 0 -1]')
    
    if direction=='x':
        direction_str=' -1 , 0, 0'
    elif direction=='y':
        direction_str=' 0 , -1, 0'
    elif direction=='z':
        direction_str=' 0 , 0, -1'
    
    
    for elset_sub in elset:
        fid.write(partname_str + elset_sub.upper() + ', GRAV, ' + putools.num.num2strf(magnitude,5) + ',' + direction_str + '\n')
            
    fid.write('**' + '\n')
        
#%%

def historyoutput(fid,options=''):
    
    '''
    *OUTPUT, HISTORY
    
    Arguments
    ------------
    fid: file identifier
    options: 
    
    Returns
    ------------
    None
    
    '''

    comma=', '
    if len(options)<=1:
        comma=''
        options=''
    
    fid.write('*OUTPUT, HISTORY' + comma + options.upper() + '\n')

#%%

def historyoutputelement(fid,variables,elset,options=''):
    
    '''
    *ELEMENT OUTPUT
    
    Arguments
    ------------
    fid: file identifier
    variables: response quantity, e.g. SF
    elset: string with element name
    
    Returns
    ------------
    None
    
    '''

    fid.write('*ELEMENT OUTPUT, ELSET=' + elset + '\n')

    if isinstance(variables,str):
        variables=[variables]
            
    for variables_sub in variables:
        fid.write(variables_sub + '\n')

                
    fid.write('**' + '\n')
    
#%%
    
def historyoutputnode(fid,variables,nset,options=''):
    
    '''
    *NODE OUTPUT
    
    Arguments
    ------------
    fid: file identifier
    variables: response quantity, e.g. 'U'
    nset: string with node name
    
    Returns
    ------------
    None
    
    '''

    fid.write('*NODE OUTPUT, NSET=' + nset + '\n')

    if isinstance(variables,str):
        variables=[variables]
            
    for variables_sub in variables:
        fid.write(variables_sub + '\n')

                
    fid.write('**' + '\n')
    
#%%

def include(fid,filename):
    
    '''
    *INCLUDE
    
    Arguments
    ------------
    fid: file identifier
    filename: string with name of inp file
    
    Returns
    ------------
    None
    
    '''

    fid.write('**' + '\n')
    fid.write('*INCLUDE, INPUT=' + filename + '\n')
    fid.write('**' + '\n')

#%%

def instance(fid,instancename,partname):
    
    '''
    *INSTANCE
    
    Arguments
    ------------
    fid: file identifier
    instancename: string with instance
    partname: string with part
    
    Returns
    ------------
    None
    
    '''
    
    fid.write('*INSTANCE, NAME=' + instancename.upper() + ', PART=' + partname.upper() + '\n')
    fid.write('**' + '\n')


#%%

def instanceend(fid):
    
    '''
    *END INSTANCE
    
    Arguments
    ------------
    fid: file identifier
    
    Returns
    ------------
    None
    
    '''
    
    fid.write('*END INSTANCE' + '\n')
    fid.write('**' + '\n')

#%%

def line(fid,line):
    
    '''
    Write line(s) of code in input file
    
    Arguments
    ------------
    fid: file identifier
    line: string or list with lines to write to file
    
    Returns
    ------------
    None
    
    '''

    if isinstance(line,str):
        line=[line]

    for line_sub in line:
        fid.write(line_sub + '\n')

#%%

def mpc(fid,id_type,nodes):
    
    '''
    *MPC
    
    Arguments
    ------------
    fid: file identifier
    id_type: e.g. 'BEAM' or 'PIN'
    nodes: array with node numbers or list with node names
    
    Returns
    ------------
    None
    
    '''

    fid.write('*MPC' + '\n')
    
    mpc_str=''
    
    if putools.num.isnumeric(nodes):
        nodes=putools.num.ensurenp(nodes)
        for nodes_sub in np.nditer(nodes):
            mpc_str=mpc_str + ',' + str(int(nodes_sub))
    elif isinstance(nodes,list):
        for nodes_sub in nodes:
            mpc_str=mpc_str + ',' + nodes_sub.upper()
        
    mpc_str=mpc_str[1:]
    fid.write(id_type +  ', ' + mpc_str + '\n')

    fid.write('**' + '\n')

#%%

def material(fid,materialname,E,v,density,alpha=None):
    
    '''
    *MATERIAL
    
    Arguments
    ------------
    fid: file identifier
    materialname: string with name
    E: elastic modulus in [Pa]
    v: Poisson ratio
    density: in [kg/m^3]
    
    Returns
    ------------
    None
    
    '''

    fid.write('*MATERIAL, NAME=' + materialname.upper() + '\n')
    fid.write('*ELASTIC' + '\n')
    putools.txt.writematrix(fid,[E,v],3,', ',['e' , 'f'])
    fid.write('*DENSITY' + '\n')
    putools.txt.writematrix(fid,[density],3,', ',['e'])
    
    if alpha is not None:
        fid.write('*EXPANSION' + '\n')
        putools.txt.writematrix(fid,[alpha],3,', ',['e'])
    
    fid.write('**' + '\n')
    fid.write('**' + '\n')


#%%

def modelchange(fid,option,elset,partname=''):
    
    '''
    *MODEL CHANGE
    
    Arguments
    ------------
    fid: file identifier
    option: 'ADD' or 'REMOVE'
    elset: string or list with element sets
    
    Returns
    ------------
    None
    
    '''

    checkarg(option,['ADD','REMOVE'])

    if isinstance(elset,str):
        elset=[elset]

    partname_str=''
    if len(partname)>0:
        partname_str=partname.upper() + '.'
    
    fid.write('*MODEL CHANGE, ' + option.upper() + '\n')

    for elset_sub in elset:
        fid.write(partname_str + elset_sub.upper() + '\n')

    fid.write('**' + '\n')

#%%
    
def node(fid,nodenumber_coord,nsetname):
    
    '''
    *NODE
    
    Arguments
    ------------
    fid: file identifier
    nodenumber_coord: array with rows [node_num,coord_x,coord_y,coord_z]
    nsetname: string with name
    
    Returns
    ------------
    None
    
    '''
    
    nodenumber_coord=np.atleast_2d(nodenumber_coord)
    
    if any(nodenumber_coord[:,0]<=0):
        putools.txt.starprint('For NSET ' + nsetname,1)
        raise Exception('***** Negative node numbers')
        
        
    if np.isnan(nodenumber_coord).any():
        putools.txt.starprint('For NSET ' + nsetname,1)
        raise Exception('***** Coordinates containing NAN')
    
    fid.write('*NODE' + ',NSET=' + nsetname.upper() + '\n')
    putools.txt.writematrix(fid,nodenumber_coord,5,',',['int' , 'f', 'f', 'f'])

    fid.write('**' + '\n')
    fid.write('**' + '\n')


#%%

def nonstructuralmass(fid,elset,unit,mass):
    
    '''
    *NONSTRUCTURAL MASS
    
    Arguments
    ------------
    fid: file identifier
    elset: string with name
    unit: MASS PER LENGTH or TOTAL MASS
    mass: magnitude 
    
    Returns
    ------------
    None
    
    '''

    fid.write('*NONSTRUCTURAL MASS, ELSET=' + elset.upper() + ', UNITS=' + unit.upper() + '\n')
    
    putools.txt.writematrix(fid,mass,5,',','e')
    
    fid.write('**' + '\n')

#%%

def nset(fid,nsetname,nodes,option=''):
    
    '''
    
    
    Arguments
    ------------
    fid: file identifier
    nsetname: string with name
    nodes: array with numbers or list with node set names
    
    Returns
    ------------
    None
    
    '''

    comma=', '
    if len(option)<=1:
        comma=''
        option=''
        
    if isinstance(nsetname,list):
        nsetname=nsetname[0]
        
    fid.write('*NSET, NSET=' + nsetname.upper() + comma + option.upper() + '\n')
    
    if putools.num.isnumeric(nodes):
        nodes=np.atleast_1d(putools.num.ensurenp(nodes))
        bins=putools.num.rangebin(len(nodes),16)
            
        for bins_sub in bins:
            putools.txt.writematrix(fid,nodes[bins_sub],'',',','int')
                
    elif isinstance(nodes,list):
        for nodes_sub in nodes:
            fid.write(nodes_sub.upper() + '\n')
    
    fid.write('**' + '\n')
    fid.write('**' + '\n')
    


#%%

def orientation(fid,name,defi,node_data):

    '''
    *ORIENTATION
    
    Arguments
    ------------
    fid: file identifier
    def: NODES
    
    Returns
    ------------
    None
    
    '''
    
    checkarg(defi, ['nodes','coordinates'])
    
    fid.write('*ORIENTATION, NAME=' + name.upper() + ', DEFINITION=' + defi.upper() + '\n')

    if defi.upper()=='NODES':
        putools.txt.writematrix(fid,node_data,'',',',['int'])
    elif defi.upper()=='COORDINATES':
        putools.txt.writematrix(fid,node_data,5,',','e')
        
    fid.write('**' + '\n')
    fid.write('**' + '\n')

#%%

def parameter(fid,parameternames,values):
    
    '''
    *PARAMETER
    
    Arguments
    ------------
    fid: file identifier
    parameternames: string or list with names
    values: array with numbers
    
    Returns
    ------------
    None
    
    '''

    if isinstance(parameternames,str):
        parameternames=[parameternames]

    if len(values)==1:
        values=values*np.ones(len(parameternames))
    
    fid.write('*PARAMETER' + '\n')
    
    for k in np.arange(len(parameternames)):
        fid.write( parameternames[k] + '=' + putools.num.num2stre(values[k],5,',') + '\n')
    
    fid.write('**' + '\n')

#%%

def part(fid,partname):
    
    '''
    *PART
    
    Arguments
    ------------
    fid: file identifier
    partname: string with name
    
    Returns
    ------------
    None
    
    '''

    fid.write('*PART, NAME=' + partname.upper() + '\n')
    fid.write('**' + '\n')

#%%

def partend(fid):
    
    '''
    *END PART
    
    Arguments
    ------------
    fid: file identifier
    
    Returns
    ------------
    None
    
    '''
    
    fid.write('*END PART' + '\n')
    fid.write('**' + '\n')

#%%

def release(fid,elset,end_id,release_id):
    
    '''
    *RELEASE
    
    Arguments
    ------------
    fid: file identifier
    elset: string or list with element set name
    end_id: string or list with end ids, e.g. 'S1' 'S2'
    release_id: string or list with end ids
    
    Returns
    ------------
    None
    
    '''
    
    # M1: refers to the rotation about the n1-axis,
    # M2 refers to the rotation about the n2-axis,
    # M1-M2: refers to a combination of rotational degrees of freedom about the n1-axis and the n2-axis,
    # T: refers to the rotation about the t-axis,
    # M1-T: refers to a combination of rotational degrees of freedom about the n1-axis and the t-axis,
    # M2-T: refers to a combination of rotational degrees of freedom about the n2-axis and the t-axis, and
    # ALLM: represents a combination of all the rotational degrees of freedom (i.e., M1, M2, and T).
    
    if isinstance(elset,str):
        elset=[elset]
        
    if isinstance(end_id,str):
        end_id=[end_id]

    if isinstance(release_id,str):
        release_id=[release_id]

    fid.write('*RELEASE' + '\n')
    
    for i in np.arange(len(end_id)):
        for j in np.arange(len(release_id)):
        
            if putools.num.isnumeric(elset):
                elset=putools.num.ensurenp(elset)
                for elset_sub in np.nditer(elset):
                    fid.write( str(int(elset_sub)) + ', ' + end_id[i] + ', ' + release_id[j]  + '\n')
            else:
                for elset_sub in elset:
                    fid.write(elset_sub + ', ' + end_id[i] + ', ' + release_id[j]  + '\n')
                    
    fid.write('**' + '\n')

#%%

def restart(fid,id_type,step=-1,frequency=100):
    
    '''
    *RESTART
    
    Arguments
    ------------
    fid: file identifier
    id_type: 'READ' or 'WRITE'
    step: number, usually the last
    frequency: frequency
    
    Returns
    ------------
    None
    
    '''

    checkarg(id_type,['READ' , 'WRITE'])
    
    if id_type.upper()=='READ':
        freq_str=''
    else:
        freq_str=', FREQUENCY=' + str(frequency)
    
    if id_type.upper()=='WRITE':
        step_str=''
    else:
        step_str=', STEP=' + str(step)
    
    
    fid.write('*RESTART' + id_type + step_str + freq_str + '\n')
    fid.write('**' + '\n')

#%%

def shellgrid(fid,elsetname,nsetname,material,thickness,x0,y0,x1,y1,z0,nx,ny,node_num,offs_y,offset='0'):
    
    dx=(x1-x0)/nx
    dy=(y1-y0)/ny
    
    x_prime=np.linspace(x0,x1,nx+1)
    y_prime=np.linspace(y0,y1,ny+1)
    
    node_matrix=np.zeros((0,4))
    for j in range(ny+1):
        x=x_prime
        y=y_prime[j]*np.ones_like(x)
        z=z0*np.ones_like(x)
        
        n=np.arange(1,nx+1+1)+offs_y*(j+1)+node_num
        node_tmp=np.column_stack((n,x,y,z))
        node_matrix=np.vstack((node_matrix,node_tmp))
        
            
    el_matrix=np.zeros((0,5))
        
    for j in range(ny):
            
        n1=np.arange(1,nx+1)+offs_y*(j+1)
        n2=n1+1
        n3=n2+offs_y
        n4=n3-1
        e=n1
            
        el_tmp=np.column_stack((e,n1,n2,n3,n4))+node_num
        el_matrix=np.vstack((el_matrix,el_tmp))
            
            
    node(fid,node_matrix,nsetname)         
    element(fid,el_matrix,'S4',elsetname)
    shellsection(fid,elsetname,material,'OFFSET=' + offset,[thickness,5])
    
    return node_matrix,el_matrix

#%%

def shellsection(fid,elset,material,options,shellproperties):
    
    '''
    *SHELL SECTION
    
    Arguments
    ------------
    fid: file identifier
    elset: string with element set name
    material: string with material set name
    options: e.g.  OFFSET=SNEG
    shellproperties: array with [thickness,number of integration points]
    
    Returns
    ------------
    None
    
    '''
    
    fid.write('*SHELL SECTION, ELSET=' + elset.upper() + ',' + ' MATERIAL=' + material.upper() + ', ' + options.upper() + '\n')

    putools.txt.writematrix(fid,shellproperties,3,',',['f','int'])

    fid.write('**' + '\n')

#%%

def shearcenter(fid,x1,x2):
    
    '''
    *SHEAR CENTER
    
    Arguments
    ------------
    fid: file identifier
    x1: offset in 1-dir in [m]
    x2: offset in 2-dir in [m]
    
    Returns
    ------------
    None
    
    '''
 
    fid.write('*SHEAR CENTER' + '\n')
    putools.txt.writematrix(fid,[x1,x2],3,',','e')
    fid.write('**' + '\n')
    
#%%

def spring(fid,elset,element_nodenumber,dofno,springstiffness):

    '''
    *ELEMENT, TYPE=SPRING2
    *SPRING
    
    Arguments
    ------------
    fid: file identifier
    elset: string with element set name
    element_nodenumber: [el,node1,node2]
    dofno: DOF 1-6
    springstiffness: in N/m
    
    Returns
    ------------
    None
    
    '''

    #*ELEMENT, TYPE=SPRING2, ELSET=SADLESPRING_X
    #301001, 100075, 10051
    #301002, 200075, 10169
    #301003, 110075, 20051
    #301004, 210075, 20169
    #*SPRING,ELSET=SADLESPRING_X
    # 1,1
    # 1.00000e+12
    
        
    fid.write('*ELEMENT, TYPE=SPRING2, ELSET=' + elset.upper() + '\n')

    putools.txt.writematrix(fid,element_nodenumber,'',',','int')

    fid.write('*SPRING, ELSET=' + elset.upper() + '\n')
    
    putools.txt.writematrix(fid,[dofno,dofno],'',',','int')

    if putools.num.isnumeric(springstiffness):
        putools.txt.writematrix(fid,springstiffness,5,',','e')
    

    fid.write('**' + '\n')
    fid.write('**' + '\n')
    
#%%

def springa(fid,elset,element_nodenumber,springstiffness):

    '''
    *ELEMENT, TYPE=SPRINGA
    *SPRING    
    
    Arguments
    ------------
    fid: file identifier
    elset: string with element set name
    element_nodenumber: [Elno,Nodeno1,Nodeno2]
    springstiffness: in N/m
    
    Returns
    ------------
    None
    
    '''
        
    fid.write('*ELEMENT, TYPE=SPRINGA, ELSET=' + elset.upper() + '\n')

    putools.txt.writematrix(fid,element_nodenumber,'',',','int')

    fid.write('*SPRING,ELSET=' + elset.upper() + '\n')

    if putools.num.isnumeric(springstiffness):
        putools.txt.writematrix(fid,springstiffness,5,',','e')
    #elif isinstance(springstiffness,list):
        #fid.write( springstiffness + '\n')

    fid.write('**' + '\n')
    fid.write('**' + '\n')

#%%
    
def static(fid,time):

    '''
    *STATIC    
    
    Arguments
    ------------
    fid: file identifier
    time: string or array with [Initial Total Min Max]
    
    Returns
    ------------
    None
    
    '''

    fid.write('*STATIC \n')

    if putools.num.isnumeric(time):
        putools.txt.writematrix(fid,time,'1',',','e')
    elif isinstance(time,str):
        fid.write(time +' \n')
    
    fid.write('**' + '\n')

#%%

def step(fid,options='',comment_str=''):

    '''
    *STEP    
    
    Arguments
    ------------
    
    Returns
    ------------
    None
    
    '''

    comma=', '
    if len(options)<1:
        comma=''
        options=''
    
    comment(fid,comment_str)
    fid.write('*STEP ' + comma + options.upper() + '\n')
    
#%%

def stepend(fid):

    '''
    *END STEP
    
    Arguments
    ------------
    fid: file identifier
    
    Returns
    ------------
    None
    
    '''
    
    fid.write('*END STEP' + '\n')
    fid.write('**' + '\n')
    fid.write('**' + '\n')
    
#%%

def surface(fid,surfname,stype,name,idf):

    '''
    *SURFACE
    
    Arguments
    ------------
    fid: file identifier
    name: name of element set
    stype: surface type, e.g. ELEMENT
    idf: edge or face identifier, e.g. SPOS
    
    Returns
    ------------
    None
    
    '''
    
    fid.write('*SURFACE, NAME=' + surfname.upper() + ', TYPE=' + stype.upper() + '\n')
    
    if stype.upper()=='ELEMENT':
        fid.write(name.upper() + ', ' + idf.upper() + '\n')
    elif stype.upper()=='NODE':
        fid.write(name.upper() + '\n')        
    
    fid.write('**' + '\n')    
    

    
#%%

def temperature(fid,op,nset,magnitude_temp,partname=''):

    '''
    *TEMPERATURE
    
    Arguments
    ------------
    fid: file identifier
    op: 'MOD' or 'NEW' for new (erase all old) or modified TEMP
    nset: string with node name
    magnitude_temp: in K
    
    Returns
    ------------
    None
    
    '''

    checkarg(op,['MOD','NEW'])

    fid.write('*TEMPERATURE, OP=' + op.upper() + '\n')
    
    partname_str=''
    if len(partname)>0:
        partname_str=partname.upper() + '.'

    if isinstance(nset,str):
        nset=[nset]
        
    magnitude_temp=putools.num.ensurenp(magnitude_temp)
    
    magnitude_temp=np.atleast_1d(magnitude_temp)
    
    if len(magnitude_temp)==1:
        magnitude_temp=magnitude_temp*np.ones([1,len(nset)])
        
    if isinstance(nset,list):
        for k,nset_sub in enumerate(nset):
           fid.write( partname_str + nset_sub + ', ' + putools.num.num2stre(magnitude_temp[k],3) + '\n')
           
    fid.write('**' + '\n')
    fid.write('**' + '\n')
    
    
 #%%

def tie(fid,tiename,adjust,postol,slavename,mastername):

    '''
    *TIE
    
    Arguments
    ------------
    fid: file identifier
    tiename: string with name
    adjust: 'YES' or 'NO' adjustment of slave node
    postol: in m, nodes outside will not be tied
    slavename: name of surface
    mastername: name of surface
    
    Returns
    ------------
    None
    
    '''
    
    checkarg(adjust,['YES','NO'])
    
    fid.write('*TIE, NAME=' + tiename.upper() + ', ADJUST=' + adjust.upper() + ', ' 'POSITION TOLERANCE=' + putools.num.num2stre(postol,3) + '\n')
    fid.write(slavename.upper() + ', ' + mastername.upper() + '\n')
    
    fid.write('**' + '\n')

#%%
   
def transverseshearstiffness(fid,k23,k13):

    '''
    *TRANSVERSE SHEAR STIFFNESS
    
    Arguments
    ------------
    fid: file identifier
    k23: shear stiffness in 2-dir
    k13: shear stiffness in 1-dir
    
    Returns
    ------------
    None
    
    '''
    
    fid.write('*TRANSVERSE SHEAR STIFFNESS' + '\n')
    putools.txt.writematrix(fid,[k23,k13],3,',','e')

    





from sys import path
import numpy as np
import odbAccess
from abaqusConstants import *
import os
from textRepr import *
from numpy import genfromtxt
import visualization
import animation
import displayGroupOdbToolset as dgo 

def p(pstring):
    prettyPrint(pstring)

# 

foldername_print=r'C:\Cloud\OD_OWP\Work\Projects\Long_span_bridges\Langenuen\Abaqus\PlotModel'

cameraPosition=(-1250, -900, 600)
cameraUpVector=(0.43, 0.43, 0.78)
nearPlane=1338
cameraTarget= (200, 0, 100)
fieldOfViewAngle=80
viewOffsetX=-100
viewOffsetY=-50

initialcolor_model='#0009FF'
initialcolor_modes='#BDBDBD'

mode_print=np.array([1,2,3,4,5,6,    7,8,9])
scalefactor=np.array([-1 , 1, 1, 1, 1 , 0.8     , 1, 1,1])*1e2

filename_model='Langenuen_model'
filename_mode='Langenuen_mode_'

png_size_model=(2000, 560)
png_size_modes=(1000, 280)

# Viewport
viewport_obj = session.viewports[session.currentViewportName]
odb_obj=viewport_obj.displayedObject

viewport_obj.viewportAnnotationOptions.setValues(
	triad=OFF, title=OFF, state=OFF, compass=OFF, legend=OFF,
	legendBox=0,
	legendDecimalPlaces= 2,
	triadPosition= (3, 4),
	legendFont='-*-verdana-medium-r-normal-*-*-80-*-*-p-*-*-*', 
	titleFont='-*-verdana-medium-r-normal-*-*-120-*-*-p-*-*-*', 
	stateFont='-*-verdana-medium-r-normal-*-*-80-*-*-p-*-*-*')
	

# Set last step
stepkeys=odb_obj.steps.keys()

viewport_obj.odbDisplay.setFrame(step=stepkeys[-1], frame=0)
firstframe=odb_obj.steps[stepkeys[-1]].frames[0]
dispfield=firstframe.fieldOutputs['U']
viewport_obj.odbDisplay.setPrimaryVariable(field=dispfield,outputPosition=NODAL,refinement=(INVARIANT, 'Magnitude'))

# View
viewport_obj.odbDisplay.basicOptions.setValues(beamScaleFactor=1.0, renderBeamProfiles=OFF, shellScaleFactor=1.0, renderShellThickness=OFF,
	referencePoints=OFF)
viewport_obj.odbDisplay.commonOptions.setValues(deformationScaling=UNIFORM, uniformScaleFactor=1,visibleEdges=NONE)
viewport_obj.odbDisplay.display.setValues(plotState=(UNDEFORMED)) #UNDEFORMED

viewport_obj.view.setValues(cameraPosition=cameraPosition)
viewport_obj.view.setValues(cameraUpVector=cameraUpVector)
viewport_obj.view.setValues(nearPlane=nearPlane)
viewport_obj.view.setValues(cameraTarget=cameraTarget)
viewport_obj.view.setValues(fieldOfViewAngle=fieldOfViewAngle)
viewport_obj.view.setValues(viewOffsetX=viewOffsetX)
viewport_obj.view.setValues(viewOffsetY=viewOffsetY)


# Print model with initial color
viewport_obj.odbDisplay.commonOptions.setValues(deformationScaling=UNIFORM, uniformScaleFactor=0,visibleEdges=NONE)
viewport_obj.setColor(initialColor=initialcolor_model)

# Set size
session.pngOptions.setValues(imageSize=png_size_model)

session.printToFile(fileName=foldername_print + '/' +filename_model,format=PNG, canvasObjects=(viewport_obj, ))


# Print modes
viewport_obj.setColor(initialColor=initialcolor_modes)
viewport_obj.odbDisplay.display.setValues(plotState=(CONTOURS_ON_DEF,UNDEFORMED))

viewport_obj.odbDisplay.superimposeOptions.setValues(edgeColorFillShade=initialcolor_modes,renderStyle=WIREFRAME) 
viewport_obj.odbDisplay.contourOptions.setValues(spectrum='Blue to red',numIntervals=20, maxAutoCompute=ON, minAutoCompute=ON) 

# Set size
session.pngOptions.setValues(imageSize=png_size_modes)

# Loop over selected modes
for k in range(len(mode_print)):
	
	viewport_obj.odbDisplay.setFrame(step=stepkeys[-1], frame=mode_print[k])
	# viewport_obj.odbDisplay.setDeformedVariable(dispfield)
	viewport_obj.odbDisplay.commonOptions.setValues(deformationScaling=UNIFORM, uniformScaleFactor=scalefactor[k])
    
	session.printToFile(foldername_print + '/' + filename_mode + str(mode_print[k]),format=PNG, canvasObjects=(viewport_obj, ))




# -*- coding: utf-8 -*-
"""
Created on 

"""

#%%

import numpy as np
import sys
sys.path.append('C:/Cloud/OD_OWP/Work/Python/Github')

from abaqustools import kw
from abaqustools import abq

#%%  Open file

foldername=r'C:\Cloud\OD_OWP\Work\Python\Github\abaqustools\example\truss'
input_filename='simple_truss.inp'
jobname='simple_truss'

fid=open(input_filename,'w')

#%%  Geometry

kw.part(fid,'part_truss')

# Lower nodes
x1=np.array([0,5,10,15,20])
y1=x1*0
z1=x1*0

nodenumber1=np.arange(1,len(x1)+1)

kw.node(fid,np.column_stack((nodenumber1,x1,y1,z1)),'NODES_BOTTOM')

# Upper nodes
x2=np.array([2.5,7.5,12.5,17.5])
y2=x2*0
z2=5*np.ones(np.shape(x2))

nodenumber2=np.arange(1,len(x2)+1)+10

kw.node(fid,np.column_stack((nodenumber2,x2,y2,z2)),'NODES_TOP')

# Elements and nodes
nodes_el=np.array([
    [100,1,2],
    [101,2,3],
    [102,3,4],
    [103,4,5],
    [201,11,12],
    [202,12,13],
    [203,13,14],
    [301,1,11],
    [302,11,2],
    [303,2,12],
    [304,12,3],
    [305,3,13],
    [306,13,4],
    [307,4,14],
    [308,14,5],
])

kw.element(fid,nodes_el,'B31','Truss_elements')

kw.beamgeneralsection(fid, 'Truss_elements', 7850, [1e-4,1e-6,0,1e-6,1e-6], [0,1,0], [210e9,81e9])

kw.release(fid, [301,302,303,304,305,306,307,308], ['S1','S2'], ['M1','M2'])

kw.nset(fid,'support',[1,5])

#%% 

kw.partend(fid)
    
kw.comment(fid,'ASSEMBLY',True)
    
kw.assembly(fid,'assembly_truss')
    
kw.instance(fid,'part_truss','part_truss')
    
kw.instanceend(fid)
    
kw.assemblyend(fid)
    
#%%  Step static

kw.step(fid,'NLGEO=NO, NAME=STEP_STATIC','Dead load')

kw.static(fid,'1e-3, 1, 1e-6, 1')

kw.gravload(fid,'new',[''],9.81)
    
kw.boundary(fid,'new','support',[1,6,0],'part_truss')

kw.boundary(fid,'new','NODES_BOTTOM',[2,2,0],'part_truss')
kw.boundary(fid,'new','NODES_TOP',[2,2,0],'part_truss')


kw.fieldoutput(fid,'NODE',['U' , 'RF' , 'COORD'],'','FREQUENCY=100')
kw.fieldoutput(fid,'ELEMENT',['SF'],'','FREQUENCY=100')
    
kw.stepend(fid)

#%%  Step modal analysis

kw.step(fid,'NAME=STEP_MODAL','')
kw.frequency(fid,50,'displacement')
    
kw.fieldoutput(fid,'NODE',['U' , 'COORD'],'','')
kw.fieldoutput(fid,'ELEMENT',['SF'],'','')

kw.stepend(fid)

#%%  Close file

fid.close()

#%%  Run job
    
# Check input file for duplicate node or element numbers
abq.checkduplicate(input_filename)
    
abq.runjob(foldername,input_filename,jobname)






# -*- coding: utf-8 -*-

#%%

import sys
import os
import time
import numpy as np
import putools
import h5py
from timeit import default_timer as timer

#%%

def static(folder_odb,jobname,folder_save,folder_python,variables=None,stepnumber=None,framenumber=None,deletetxt=True,saveh5=True,prefix=None,postfixh5='_exportstatic',exportscript=None):

    '''
    Export static results from odb file

    Arguments
    ------------
    folder_odb (str): folder of odb file
    jobname (str): name of odb file
    folder_save (str): folder for export
    folder_python (str): folder where the python abaqustools repo is located (must be added in abaqus)
    variables (list): output quantities to export
    stepnumber (int): step number 
    framenumber (int or list): frame number(s)
    deletetxt (bool): delete txt output files after export
    saveh5 (bool): save output to h5 file
    prefix (str): prefix for txt files
    postfixh5 (str): postfix for h5 file
    exportscript (str): name of python script that is created
    
    Returns
    ------------
    None
    
    '''
    
    # Stepnumber
    if stepnumber is None:
        stepnumber=-1
    
    # Framenumber
    if framenumber is None:
        framenumber=-1

    # Variables to export
    if variables is None:
       variables=['u' , 'sf' , 'nodecoord' , 'elconn' , 'elset']

    # Script py file
    if exportscript is None:
       exportscript=jobname + '_exportstatic'
    
    # Run main
    exportmain(folder_odb,jobname,folder_save,folder_python,variables,stepnumber,framenumber,deletetxt=deletetxt,saveh5=saveh5,prefix=prefix,postfixh5=postfixh5,exportscript=exportscript)

def modal(folder_odb,jobname,folder_save,folder_python,variables=None,stepnumber=None,framenumber=None,deletetxt=True,saveh5=True,prefix=None,postfixh5='_exportmodal',exportscript=None,nodes=None):
    
    '''
    Export modal results from odb file

    Arguments
    ------------
    folder_odb (str): folder of odb file
    jobname (str): name of odb file
    folder_save (str): folder for export
    folder_python (str): folder where the python abaqustools repo is located (must be added in abaqus)
    variables (list): output quantities to export
    stepnumber (int): step number 
    framenumber (int or list): frame number(s)
    deletetxt (bool): delete txt output files after export
    saveh5 (bool): save output to h5 file
    prefix (str): prefix for txt files
    postfixh5 (str): postfix for h5 file
    exportscript (str): name of python script that is created
    nodes (list): selected nodes to export
    
    Returns
    ------------
    None
    
    '''
    
    if stepnumber is None:
        stepnumber=-1
    
    # Framenumber
    if framenumber is None:
        framenumber='skipfirst'
    
    # Variables to export
    if variables is None:
       variables=['f' , 'gm' , 'phi' , 'phi_sf' , 'nodecoord' , 'elconn']

    # Script py file
    if exportscript is None:
       exportscript=jobname + '_exportmodal'
       
    # Run main
    exportmain(folder_odb,jobname,folder_save,folder_python,variables,stepnumber,framenumber,deletetxt=deletetxt,saveh5=saveh5,prefix=prefix,postfixh5=postfixh5,exportscript=exportscript,nodes=nodes)

#%%

def exportmain(folder_odb,jobname,folder_save,folder_python,variables,stepnumber,framenumber,deletetxt=True,saveh5=True,prefix=None,postfixh5=None,exportscript=None,nodes=None):

    '''
    Export outputs from odb file to txt or h5 file

    Arguments
    ------------
    folder_odb (str): folder of odb file
    jobname (str): name of odb file
    folder_save (str): folder for export
    folder_python (str): folder where the python abaqustools repo is located (must be added in abaqus)
    variables (list): output quantities to export
    stepnumber (int): step number 
    framenumber (int or list): frame number(s)
    deletetxt (bool): delete txt output files after export
    saveh5 (bool): save output to h5 file
    prefix (str): prefix for txt files
    postfixh5 (str): postfix for h5 file
    exportscript (str): name of python script that is created
    
    Returns
    ------------
    None
    
    '''
    
    # Cut odb extension
    if jobname.endswith('.odb'):
        jobname=jobname[:-4]
    
    # Prefix for saving txt files
    if prefix is None:
        prefix=jobname+'_export_'
    
    # Postfix for h5 file
    if postfixh5 is None:
        postfixh5=jobname+'_export'
    
    # Check if ODB file is found
    file_name=folder_odb + '\\' + jobname + '.odb'
    file_exist_logic=os.path.isfile(file_name)
    if not file_exist_logic:
        print('***** ' + file_name)
        raise Exception('***** ODB file not found')
        
    # Check if python folder and python files are given correctly
    file_name=folder_python + '\\' + 'odbfunc.py'
    file_exist_logic=os.path.isfile(file_name)
    if not file_exist_logic:
        print('***** ' + folder_python)
        raise Exception('***** Supplied python (odbexport) folder and odbfunc module not found, please check that "folder_python" is correctly specified')    
 
    # Ensure list
    if isinstance(variables,str):
        variables=[variables]
        
    # Lower case
    variables=[x.lower() for x in variables]

    # Check input variables
    variables_allowed= ['u' , 'sf' , 'phi' , 'phi_sf' , 'f' , 'gm' , 'nodecoord' , 'elconn' , 'elset']
    for k in np.arange(len(variables)):
        if not variables[k] in variables_allowed:
            print('***** Check variable ' + variables[k])
            raise Exception('***** Variable identifier not allowed')
            
    # Write py-file for export
    writepyscript(folder_odb,jobname,folder_save,folder_python,variables,stepnumber,framenumber,exportscript=exportscript,prefix=prefix,nodes=nodes)
    
    # Run py-file in abaqus
    system_cmd='abaqus cae noGUI=' + folder_save + '/' + exportscript + '.py'
    
    t0=timer()
    sys_out=os.popen(system_cmd).read()
    t1=timer()
    
    idx_error=sys_out.find('error')
        
    if idx_error>0:
        print(sys_out)
        raise Exception('***** Export aborted due to ABAQUS errors, see above')
    else:
        print('***** ABAQUS txt export completed in ' + putools.num.num2strf(t1-t0,1) + ' s')
        
    
    time.sleep(0.5)
    
    # Check that txt files are generated
    for k in np.arange(len(variables)):
         
        # Base for txt files
        filename_base=folder_save + '/' + prefix      
        
        # Check that txt files have been created
        do_exist=os.path.exists(filename_base+variables[k]+'.txt')
        if do_exist==False:
            print('***** ' + 'File ' + filename_base+variables[k] + '.txt' + ' not found')
            
     
    # Save h5 file        
    if saveh5==True:

        # Base for txt files
        hf_name=folder_save + '/' + jobname + postfixh5 + '.h5'
        
        # Overwrite file
        if os.path.exists(hf_name):
            os.remove(hf_name)
            time.sleep(1)
            
        hf = h5py.File(hf_name,'w')
        dt = h5py.special_dtype(vlen=str)
            
        for k in np.arange(len(variables)):
            
            # Base for txt files
            filename_base=folder_save + '/' + prefix      
            
            # Check if file exists
            
            do_exist=os.path.exists(filename_base+variables[k]+'.txt')
            if do_exist==False:
                print('***** ' + 'File ' + filename_base+variables[k] + '.txt' + ' not found, skipping')
                print('***** ' + 'This means that variable ' + variables[k] + ' will not be exported to h5 file')
                continue
            
            # Read txt file
            data_from_txt=np.genfromtxt(filename_base+variables[k]+'.txt', delimiter=',')
            
            if variables[k]=='u':
            
                hf.create_dataset('u',data=data_from_txt)
                hf['u'].attrs.modify('Type','Displacement')
                hf['u'].attrs.modify('Unit','m, rad')
                
                fid=open(filename_base+'u_label.txt','r'); label=fid.read().splitlines(); fid.close()  
                hf.create_dataset('u_label',data=label,dtype=dt)
                hf['u_label'].attrs.modify('Type','DOF labels')
                
            elif variables[k]=='sf':
            
                hf.create_dataset('sf',data=data_from_txt)
                hf['sf'].attrs.modify('Type','Section force')
                hf['sf'].attrs.modify('Unit','m, rad')
                
                fid=open(filename_base+'sf_label.txt','r'); label=fid.read().splitlines(); fid.close()  
                hf.create_dataset('sf_label',data=label,dtype=dt)
                hf['sf_label'].attrs.modify('Type', 'Element labels')
                
            elif variables[k]=='phi':
            
                hf.create_dataset('phi',data=data_from_txt)
                hf['phi'].attrs.modify('Type','Modal displacement')
                hf['phi'].attrs.modify('Unit','m, rad')
                
                fid=open(filename_base+'phi_label.txt','r'); label=fid.read().splitlines(); fid.close()  
                hf.create_dataset('phi_label',data=label,dtype=dt)
                hf['phi_label'].attrs.modify('Type','DOF labels')
                
            elif variables[k]=='phi_sf':
            
                hf.create_dataset('phi_sf',data=data_from_txt)
                hf['phi_sf'].attrs.modify('Type','Modal section force')
                hf['phi_sf'].attrs.modify('Unit','N, Nm')
                
                fid=open(filename_base+'phi_sf_label.txt','r'); label=fid.read().splitlines(); fid.close()  
                hf.create_dataset('phi_sf_label',data=label,dtype=dt)
                hf['phi_sf_label'].attrs.modify('Type','Element labels')        
                
            elif variables[k]=='f':
            
                hf.create_dataset('f',data=data_from_txt)
                hf['f'].attrs.modify('Type','Natural frequency')
                hf['f'].attrs.modify('Unit','Hz')
                
            elif variables[k]=='gm':
            
                hf.create_dataset('gm',data=data_from_txt)
                hf['gm'].attrs.modify('Type','Generalized mass')
                hf['gm'].attrs.modify('Unit','kg')
                
            elif variables[k]=='nodecoord':
            
                hf.create_dataset('nodecoord',data=data_from_txt)
                hf['nodecoord'].attrs.modify('Type','Node coordinate')
                hf['nodecoord'].attrs.modify('Unit','Node number, m')     
                
            elif variables[k]=='elconn':
            
                hf.create_dataset('elconn',data=data_from_txt)
                hf['elconn'].attrs.modify('Type','Element connectivity')
                hf['elconn'].attrs.modify('Unit','Element number, Node number')                     
                
            elif variables[k]=='elset':
           
                hf.create_dataset('elset',data=data_from_txt)
                hf['elset'].attrs.modify('Type','Element sets')
                hf['elset'].attrs.modify('Unit','Element number')      
           
                fid=open(filename_base+'elset_label.txt','r'); label=fid.read().splitlines(); fid.close()  
                hf.create_dataset('elset_label',data=label,dtype=dt)
                hf['elset_label'].attrs.modify('Type','Element labels')    
                            
                
    if saveh5==True:
        hf.close()
    
    # Delete txt files identified by prefix
    if deletetxt==True:
        deletefiles(folder_save,prefix,['.txt'])
        
    # Delete rpy files
    deletefiles(folder_odb,'abaqus.rpy',[''])
        

#%% 

def deletefiles(foldername,name_match,extensions):
    
    '''
    Delete files in folder that match search criteria

    Arguments
    ------------
    foldername (str): folder to search
    name_match (str): file name to delete (partial match allowed)
    extensions (list): specified extensions to include
    
    Returns
    ------------
    None
    
    '''

    # Find files that match file name 
    
    filenames_dir=os.listdir(foldername)
    idx_match=putools.num.listindexsub(filenames_dir,name_match)
    
    for k in np.arange(len(idx_match)):
            
        filename_remove=filenames_dir[idx_match[k]]
        
        match_ext=[filename_remove.endswith(ext) for ext in extensions]
        
        # Only files with specified extensions
        if any(match_ext):
        
            try:
                os.remove(foldername + '/' + filename_remove)
            except:
                print('***** Unable to delete file ' + foldername + '/' + filename_remove)
                    
            

#%% Export modal script

def writepyscript(folder_odb,jobname,folder_save,folder_python,variables,stepnumber,framenumber,exportscript='Export',prefix='_export',nodes=None):
    
    '''
    Write py script that for export within Abaqus environment

    Arguments
    ------------
    folder_odb (str): folder of odb file
    jobname (str): name of odb file
    folder_save (str): folder where output files are saved
    folder_python (str): folder where the python abaqustools repo is located (must be added in abaqus)
    folder_python (str): folder where the python abaqustools repo is located (must be added in abaqus)
    variables (list): output quantities to export
    stepnumber (int): step number 
    framenumber (int or list): frame number(s)
    exportscript (str): name of python script that is created
    prefix (str): prefix for txt files

    Returns
    ------------
    None
    
    '''
        
    # Ensure list
    if isinstance(variables,str):
        variables=[variables]
    
    # Format folder
    folder_odb=folder_odb.replace('\\','/')
    folder_save=folder_save.replace('\\','/')
    
    lines=['']
    
    lines.append('import os')
    lines.append('import sys')
    lines.append('import numpy as np')
    lines.append('')
    
    lines.append('folder_odb=' + '\'' + folder_odb + '\'')
    lines.append('jobname=' + '\'' + jobname + '\'')
    lines.append('folder_save=' + '\'' + folder_save + '\'')
    lines.append('folder_python=' + '\'' + folder_python + '\'')
    
    lines.append('prefix=' + '\'' + prefix + '\'')
    lines.append('')
    
    lines.append('# Import functions for export (odbexport package)')
    lines.append('sys.path.append(folder_python)')
    lines.append('import odbfunc')
    lines.append('')
    
    lines.append('# Open ODB')
    lines.append('odb_id=odbfunc.open_odb(folder_odb,jobname)')
    lines.append('')
    
    if isinstance(stepnumber,str):
        stepnumber_str=stepnumber
    else:
        stepnumber_str=str(stepnumber)
    
    if isinstance(framenumber,str):
        framenumber_str=framenumber
        if framenumber_str=='skipfirst':
            framenumber_str='\'skipfirst\''
    else:
        framenumber_str=str(framenumber)
        

    lines.append('# Step and frames to export')
    lines.append('stepnumber=' + stepnumber_str)
    lines.append('framenumber=' + framenumber_str)
    lines.append('')
    
    lines.append('# Node numbers to export (if set to None, all will be exported)')
    
    if nodes is None:
        lines.append('nodes=' + 'None')
    else:
    
        nodes = [int(num) for num in nodes]
        lines.append('nodes=' + '[')

        for i in range(0, len(nodes), 20):
            str_nodes_sub=', '.join(map(str, nodes[i:i+20]))
            lines.append(str_nodes_sub + ',')
        
        lines.append(']')
        
    lines.append('')

    if 'f' in variables:
        lines.append('# Frequencies')
        lines.append('f=odbfunc.exporthistoryoutput(odb_id,stepnumber,' + '\'' + 'EIGFREQ' + '\'' +  ')')
        lines.append('odbfunc.save2txt(folder_save,' +  '\'f\'' + ',f,atype=1,prefix=prefix)')
        lines.append('')

    if 'gm' in variables:
        lines.append('# Generalized mass')
        lines.append('gm=odbfunc.exporthistoryoutput(odb_id,stepnumber,' + '\'' + 'GM' + '\'' +  ')')
        lines.append('odbfunc.save2txt(folder_save,' + '\'gm\'' + ',gm,atype=1,prefix=prefix)')
        lines.append('')

    if 'phi' in variables:
        lines.append('# Mode shapes')
        lines.append('(phi,phi_label)=odbfunc.exportdisplacement(odb_id,stepnumber,framenumber=framenumber,nodes=nodes' + ')')
        lines.append('odbfunc.save2txt(folder_save,' + '\'phi\'' + ',phi,atype=1,prefix=prefix)')
        lines.append('odbfunc.save2txt(folder_save,' + '\'phi_label\'' + ',phi_label,atype=2,prefix=prefix)')
        lines.append('')

    if 'phi_sf' in variables:
        lines.append('# Modal section forces')
        lines.append('(phi_sf,phi_sf_label)=odbfunc.exportsectionforce(odb_id,stepnumber,framenumber=framenumber' + ')')
        lines.append('odbfunc.save2txt(folder_save,' + '\'phi_sf\'' + ',phi_sf,atype=1,prefix=prefix)')
        lines.append('odbfunc.save2txt(folder_save,' + '\'phi_sf_label\'' + ',phi_sf_label,atype=2,prefix=prefix)')
        lines.append('')

    if 'u' in variables:
        lines.append('# Displacements')
        lines.append('(u,u_label)=odbfunc.exportdisplacement(odb_id,stepnumber,framenumber=framenumber,nodes=nodes' + ')')
        lines.append('odbfunc.save2txt(folder_save,' + '\'u\'' + ',u,atype=1,prefix=prefix)')
        lines.append('odbfunc.save2txt(folder_save,' + '\'u_label\'' + ',u_label,atype=2,prefix=prefix)')
        lines.append('')

    if 'sf' in variables:
        lines.append('# Section forces')
        lines.append('(sf,sf_label)=odbfunc.exportsectionforce(odb_id,stepnumber,framenumber=framenumber' + ')')
        lines.append('odbfunc.save2txt(folder_save,' + '\'sf\'' + ',sf,atype=1,prefix=prefix)')
        lines.append('odbfunc.save2txt(folder_save,' + '\'sf_label\'' + ',sf_label,atype=2,prefix=prefix)')
        lines.append('')
        
    if 'nodecoord' in variables:
        lines.append('# Node coordinates')
        lines.append('nodecoord=odbfunc.exportnodecoord(odb_id,stepnumber' + ',0)')
        lines.append('odbfunc.save2txt(folder_save,' + '\'nodecoord\'' + ',nodecoord,atype=1,prefix=prefix)')
        lines.append('')
        
    if 'elconn' in variables:
        lines.append('# Element connectivity')
        lines.append('elconn=odbfunc.exportelconn(odb_id)')
        lines.append('odbfunc.save2txt(folder_save,' + '\'elconn\'' + ',elconn,atype=1,prefix=prefix)')
        lines.append('')
        
    if 'elset' in variables:
        lines.append('# Element sets')
        lines.append('(elset,elset_label)=odbfunc.exportelsets(odb_id)')
        lines.append('odbfunc.save2txt(folder_save,' + '\'elset\'' + ',elset,atype=1,prefix=prefix)')
        lines.append('odbfunc.save2txt(folder_save,' + '\'elset_label\'' + ',elset_label,atype=2,prefix=prefix)')
        lines.append('')
    
    lines.append('# Close ODB')
    lines.append('odbfunc.close_odb(odb_id)')
        
    # Write py script
    fid=open(folder_save + '\\' + exportscript + '.py', 'w')
    for lines_sub in lines:
        fid.write( lines_sub + '\n')

    fid.close()





# -*- coding: utf-8 -*-


#@author: OWP


#%%

import numpy as np
import os
import putools
import time

#%%

def printerror(jobname,foldername=''):

    '''
    Search through msg or dat file for errors, print error if found
    
    Arguments
    ------------
    jobname (string): name of odb file
    foldername (string): folder of odb file
    
    Returns
    ------------
    None
    
    '''
    
    if len(foldername)>0:
        foldername=foldername + '\\'
        
    # Remove odb extension if provided
    if jobname.endswith('.odb'):
            jobname=jobname[:-4]
    
    
    filename=foldername + jobname + '.msg'
    file_exist=os.path.isfile(filename)
    
    # If msg file dont exist, try dat
    if file_exist==False:
        filename=foldername + jobname + '.dat'
        file_exist=os.path.isfile(filename)
    
    # If file dont exist, set empty
    if file_exist==True:
        fid=open(filename, 'r')
        lines=fid.read().splitlines()
        fid.close()
    else:
        lines=['']
    
    # Search for error, print if found
    search_string=' ***ERROR'
    idx_error=putools.num.listindexsub(lines,search_string)

    if len(idx_error)>0:
        putools.txt.starprint('FOUND ABAQUS ERRORS:',1)

        for k in np.arange(4):
            
            print(lines[k+idx_error[0]])
    
#%%

def checkduplicate(inputname):
    
    '''
    Check input file, alert if duplicate node or element numbers
    
    Arguments
    ------------
    inputname (string): name of input file
    
    Returns
    ------------
    None
    
    '''
    
    file_exist=os.path.isfile(inputname)
    
    # Error if input file dont exist
    if file_exist==False:
        print('***** File ' + inputname)
        raise Exception('***** File not found')
    
    fid=open(inputname, 'r')
    input_file_lines=fid.read().splitlines()
    fid.close()

    node_or_element=['*NODE,' , '*ELEMENT,']

    # Index of lines with star, used for finding end of node/element block
    idx_star=putools.num.listindexsub(input_file_lines,'*')

    for index in np.arange(2):
        
        # Find all lines with *NODE or *ELEMENT
        idx_keyword=putools.num.listindexsub(input_file_lines,node_or_element[index])
        
        numbers_list=[None]*len(idx_keyword)
    
        # Find node or element numbers 
        for k in np.arange(len(idx_keyword)):
        
            idx_star_next=next(x for x in idx_star if x > idx_keyword[k])
            
            line_range=np.arange(idx_keyword[k]+1,idx_star_next)
            input_file_lines_sub = [input_file_lines[i] for i in line_range]
            
            numeric_block=putools.num.str2num(input_file_lines_sub,'int',1)
            numbers_list[k]=numeric_block[:,0]
            
            #[float(s) for s in example_string.split(',')]
    
        # Find duplicates
        numbers_all=np.concatenate(numbers_list,axis=0)
        (u,c)= np.unique(numbers_all, return_counts=True)
        numbers_dup=u[c>1]

        if len(numbers_dup)>0:
            
            print('***** Duplicate ' + node_or_element[index] + ' numbers:')
            print(numbers_dup)
            raise Exception('***** Duplicates not allowed, see above printed numbers')
            
            
#%%

def runjob(foldername,inputname,jobname='',abaqus_cmd='abaqus',cpus=4,echo_cmd=True,halt_error=True,oldjobname=''):
    
    '''
    Check input file, alert if duplicate node or element numbers
    
    Arguments
    ------------
    foldername (string): folder of input file
    inputname (string): name of input file
    jobname (string): name of odb file, if empty then equal to inputname
    abaqus_cmd (string): string supplied to system command, usually 'abaqus'
    cpus (number): number of cores
    echo_cmd (bool): echo console output from system
    halt_error (bool): halt or not if error in Abaqus analysis
    oldjobname (string): name of old odb file, only relevant for restart analysis, else set to empty
        
    Returns
    ------------
    completed (bool): if the job is completed or not
    
    
    '''        
    
    if len(abaqus_cmd)<1:
        abaqus_cmd='abaqus'
    
    if len(jobname)<1:
        jobname=inputname;
    
    if jobname[-4:].casefold()=='.inp' or jobname[-4:].casefold()=='.odb':
        jobname=jobname[:-4]
    
    print('***** Running ABAQUS job ' + jobname)
    
    # Save current folder
    folder_current=os.getcwd()
    
    # Change to folder where inputfile is
    os.chdir(foldername)
    
    # If lock file exists, delete
    file_name_lock=foldername + '\\' + jobname + '.lck'
    file_exist=os.path.isfile(file_name_lock)
    if file_exist:
        print('***** Lock-file existing, deleting')
        os.remove(file_name_lock)
        time.sleep(0.1)

    # Create system command input
    system_cmd=''
    system_cmd=system_cmd + abaqus_cmd
    system_cmd=system_cmd +' job=' + jobname
    
    # Add oldjob if provided
    if len(oldjobname)>0:
        system_cmd=system_cmd + ' oldjob=' + oldjobname
                                         
    system_cmd=system_cmd + ' input=' + inputname
    system_cmd=system_cmd + ' interactive'
    system_cmd=system_cmd + ' cpus=' + str(cpus)
    
    # [abaqus_cmd ' job=' jobname ' input=' inputname ' interactive' ' cpus=' num2str(cpus) ]

    t0=putools.timing.tic()
    #(sys_out)=os.system(system_cmd)
    sys_out=os.popen(system_cmd).read()
    t1=putools.timing.tocs(t0)
    
    # Change back to original folder
    os.chdir(folder_current)
    
    if echo_cmd:
        print(sys_out)
    
    # Check if job is successful
    completed='COMPLETED' in sys_out
    if completed:
        print('***** ABAQUS job completed in ' + putools.num.num2strf(t1,1) + ' s')
    
    error_bool='Abaqus/Analysis exited with error' in sys_out

    if error_bool:
        time.sleep(1)
        printerror(jobname,foldername)
        
        if halt_error:
            raise Exception('***** Stopped due to ABAQUS errors, see message above')
            
    return completed


# -*- coding: utf-8 -*-

#%% Import packages

import h5py    
import numpy as np    
import matplotlib.pyplot as plt
import sys

sys.path.append('C:/Cloud/OD_OWP/Work/Python/Github')

from abaqustools import odbexport

#%% Export from odb

folder_odb='C:/Cloud/OD_OWP/Work/Python/Github/abaqustools/odbexport_example'
jobname='Model_single_deck'
folder_save=folder_odb
folder_python='C:/Cloud/OD_OWP/Work/Python/Github/abaqustools/odbexport'

# Export modes to h5 file
odbexport.export.modal(folder_odb,jobname,folder_save,folder_python)

# Export static state for dead loads to h5 file
odbexport.export.static(folder_odb,jobname,folder_save,folder_python,stepnumber=-2,framenumber=-1,deletetxt=False,saveh5=False)

#%% Import modal properties

hf=h5py.File('Model_single_deck_exportmodal.h5', 'r')

# Frequencies
f=np.array(hf.get('f'))

# Generalized mass
gm=np.array(hf.get('gm'))

# Node coordinates all nodes (size [N_nodes,4])
nodecoord=np.array(hf.get('nodecoord'))

# Mode shape matrix for all DOFs (size [6*N_nodes,N_modes])
phi=np.array(hf.get('phi'))

# Labels corresponding to each row of phi (each DOF), as a list of strings
phi_label_temp=np.array(hf.get('phi_label'))
phi_label=phi_label_temp[:].astype('U10').ravel().tolist()

# The DOFs are as follows:
# U1=x=longitudinal translation (along bridge)
# U2=y=horizontal translation
# U3=z=vertical translation
# UR1=rotation about x aka. torsion
# UR2=rotation about y aka. vertical bending
# UR3=rotation about z aka. horizontal bending

#%% Get node coordinates and mode shape for bridge deck only

# Nodes in bridge deck 
node_deck=np.arange(1001,1311+1,1)

# Create list index of nodes in bridge deck
index_node_deck=[]

for k in np.arange(len(node_deck)):
    index_node_deck.append(np.argwhere(node_deck[k]==nodecoord[:,0])[0,0])
   
nodecoord_deck=nodecoord[index_node_deck,:]

   
# Create list of indices of y,z,t DOFs in bridge deck
index_y=[]
index_z=[]
index_t=[]

for k in np.arange(len(node_deck)):
    str_y=str(node_deck[k]) + '_U2'
    index_y.append(phi_label.index(str_y))  
    
    str_z=str(node_deck[k]) + '_U3'
    index_z.append(phi_label.index(str_z))  
    
    str_t=str(node_deck[k]) + '_UR1'
    index_t.append(phi_label.index(str_t))
    
    
phi_y=phi[index_y,:]
phi_z=phi[index_z,:]
phi_t=phi[index_t,:]


#%% Plot single mode

mode_plot=0 # Mode number to plot
scale_factor=1 # Scale factor

plt.figure()

x=nodecoord_deck[:,1] #x-coordinate of deck nodes

h1=plt.plot(x,phi_y[:,mode_plot]*scale_factor, label='Horizontal')
h2=plt.plot(x,phi_z[:,mode_plot]*scale_factor, label='Vertical')
h3=plt.plot(x,phi_t[:,mode_plot]*scale_factor, label='Torsion')

plt.xlabel('x [m]')
plt.ylabel('Modal deflection [m or rad]')

plt.legend()

plt.show()




import os
import sys
import numpy as np

folder_odb='C:/Cloud/OD_OWP/Work/Python/Github/abaqustools/odbexport_example'
jobname='Model_single_deck'
folder_save='C:/Cloud/OD_OWP/Work/Python/Github/abaqustools/odbexport_example'
folder_python='C:/Cloud/OD_OWP/Work/Python/Github/abaqustools/odbexport'
prefix='Model_single_deck_export_'

# Import functions for export (odbexport package)
sys.path.append(folder_python)
import odbfunc

# Open ODB
odb_id=odbfunc.open_odb(folder_odb,jobname)

# Step and frames to export
stepnumber=-2
framenumber=-1

# Node numbers to export (if set to None, all will be exported)
nodes=None

# Displacements
(u,u_label)=odbfunc.exportdisplacement(odb_id,stepnumber,framenumber=framenumber,nodes=nodes)
odbfunc.save2txt(folder_save,'u',u,atype=1,prefix=prefix)
odbfunc.save2txt(folder_save,'u_label',u_label,atype=2,prefix=prefix)

# Section forces
(sf,sf_label)=odbfunc.exportsectionforce(odb_id,stepnumber,framenumber=framenumber)
odbfunc.save2txt(folder_save,'sf',sf,atype=1,prefix=prefix)
odbfunc.save2txt(folder_save,'sf_label',sf_label,atype=2,prefix=prefix)

# Node coordinates
nodecoord=odbfunc.exportnodecoord(odb_id,stepnumber,0)
odbfunc.save2txt(folder_save,'nodecoord',nodecoord,atype=1,prefix=prefix)

# Element connectivity
elconn=odbfunc.exportelconn(odb_id)
odbfunc.save2txt(folder_save,'elconn',elconn,atype=1,prefix=prefix)

# Element sets
(elset,elset_label)=odbfunc.exportelsets(odb_id)
odbfunc.save2txt(folder_save,'elset',elset,atype=1,prefix=prefix)
odbfunc.save2txt(folder_save,'elset_label',elset_label,atype=2,prefix=prefix)

# Close ODB
odbfunc.close_odb(odb_id)



# -*- coding: utf-8 -*-                       
from sys import path
import numpy as np
import odbAccess
import os
from textRepr import *
from timeit import default_timer as timer

# This module is imported in Abaqus by "import odbfunc"
# The folder containing odbfunc.py must first be added to the system path

#%%

def p(pstring):
    prettyPrint(pstring)

#%%

def open_odb(folder_odb,jobname):
    
    '''
    Open ODB file

    Arguments
    ------------
    folder_odb (str): folder of odb file
    jobname (str): name of odb file

    Returns
    ------------
    odb_id (odb object): odb file identifier
    
    '''

    if jobname.endswith('.odb'):
        jobname=jobname[:-4]
    
    odb_id=odbAccess.openOdb(folder_odb + '/' + jobname + '.odb',readOnly=True)
    
    return odb_id
    
#%%

def close_odb(odb_id):

    '''
    Close ODB file

    Arguments
    ------------
    odb_id (odb object): odb file identifier

    Returns
    ------------
    None
    
    '''

    odb_id.close()

#%%

def exporthistoryoutput(odb_id,stepnumber,hist_key,assembly_name=None):
    
    '''
    Export history outputs

    Arguments
    ------------
    odb_id (odb object): odb object
    stepnumber (int): step number for export, usually -1
    hist_key (str): output to export, e.g. EIGFREQ,GM,DAMPRATIO,EIGREAL,EIGIMAG
    assembly_name (str): assembly

    Returns
    ------------
    output_vec (np array): vector with history outputs
    
    '''

    step_name=odb_id.steps.keys()[stepnumber]
    step_id=odb_id.steps[step_name]
    
    if assembly_name is None:
        assembly_keys=step_id.historyRegions.keys()
        assembly_name=assembly_keys[0]
    
    hist_output_keys=step_id.historyRegions[assembly_name].historyOutputs.keys()
    
    # If key not found, return zero
    if hist_key not in hist_output_keys:
        output_vec=np.array([0])
        return output_vec
    else:
        hist_output=step_id.historyRegions[assembly_name].historyOutputs[hist_key]
        
    output_vec=np.array([hist_output.data[j][1] for j in range(0,len(hist_output.data))])
    
    return output_vec
    
#%%

def exportdisplacement(odb_id,stepnumber,framenumber=None,nodes=None):

    '''
    Export displacement field (U,UR) for a single step and multiple frames

    Arguments
    ------------
    odb_id (odb object): odb file identifier
    stepnumber (int): step number for export, usually -1
    framenumber (list): frame number(s) for export; None gives all frames; 'skipfirst' gives all except 0
    
    Returns
    ------------
    disp_matrix: matrix with each frame as column (e.g. N_DOF*N_frames)
    label_vec: list with DOF labels of all N_DOF
    
    '''
    
    step_name=odb_id.steps.keys()[stepnumber]
    step_id=odb_id.steps[step_name]
    
    if framenumber is None:
        framenumber=range(len(step_id.frames))
    elif framenumber=='skipfirst':
        framenumber=range(1,len(step_id.frames))
    elif isinstance(framenumber,int):
        framenumber=np.array([framenumber])
    
    frame_id=step_id.frames[-1]
    if not frame_id.fieldOutputs.has_key('U'):
        disp_matrix=np.array([0])
        label_vec=np.array([0])
        return disp_matrix,label_vec
        
    
    N_node=len(frame_id.fieldOutputs['U'].values)          
    N_frame=len(framenumber)    
    
    # If nodes are not provided, export for all
    if nodes is None:
        N_node_export=N_node
        idx_export=range(N_node)
    else:
        disp_trans=frame_id.fieldOutputs['U']
        disp_trans_val=disp_trans.values
        
        nodes_all=[disp_trans_val[n].nodeLabel for n in range(N_node) ]
        
        idx_export=[nodes_all.index(x) for x in nodes if x in nodes_all] 
        N_node_export=len(idx_export)
        
    disp_matrix=np.zeros((N_node_export*6,N_frame))
    
    t_start=timer()
    for z in np.arange(len(framenumber)):
        
        frame_id=step_id.frames[framenumber[z]]
        
        disp_trans=frame_id.fieldOutputs['U']
        disp_trans_val=disp_trans.values
        
        disp_rot=frame_id.fieldOutputs['UR']
        disp_rot_val=disp_rot.values
        
        U_temp=np.array([disp_trans_val[n].data for n in idx_export ])
        UR_temp=np.array([disp_rot_val[n].data for n in idx_export ])
        
        disp_matrix[:,z]=np.hstack((U_temp,UR_temp)).flatten()
    
    label_vec_temp=[ [str(disp_trans_val[n].nodeLabel) + '_' + s  for s in disp_trans.componentLabels] + [str(disp_rot_val[n].nodeLabel) + '_' + s  for s in disp_rot.componentLabels] for n in idx_export ]
    label_vec=[item for sublist in label_vec_temp for item in sublist]
    
    t_end=timer()
    print('Time displacement ' + str(t_end-t_start) + ' s')
    return disp_matrix,label_vec

#%%

def exportnodecoord(odb_id,stepnumber,framenumber):
    
    '''
    Export node coordinates for a single step and a single frame

    Arguments
    ------------
    odb_id (odb object): odb file identifier
    stepnumber (int): step number for export, usually -1
    framenumber (int): frame number for export
    
    Returns
    ------------
    nodecoord: matrix with [nodenumber,x,y,z] as rows, size N_NODE*4
    
    '''

    step_name=odb_id.steps.keys()[stepnumber]
    step_id=odb_id.steps[step_name]
            
    frame_id=step_id.frames[framenumber]
    if 'COORD' not in frame_id.fieldOutputs.keys():
        nodecoord_matrix=np.array([0 , 0 , 0])
        nodecoord_label=np.array([0])
        nodecoord=np.hstack((nodecoord_label,nodecoord_matrix))
        return nodecoord
        
    t_start=timer()
    coord_val=frame_id.fieldOutputs['COORD'].values
    nodecoord_matrix=np.array([coord_val[ii].data for ii in range(len(coord_val))])
    nodecoord_label=np.zeros((np.shape(nodecoord_matrix)[0],1))
    nodecoord_label[:,0]=np.array([coord_val[ii].nodeLabel for ii in range(len(coord_val))])
    
    t_end=timer()
    print('Time nodecoord ' + str(t_end-t_start) + ' s')
    
    nodecoord=np.hstack((nodecoord_label,nodecoord_matrix))
    
    return nodecoord
        
#%%

def exportsectionforce(odb_id,stepnumber,framenumber=None):
    
    '''
    Export section forces (SF,SM) (beam only) for a single step and multiple frames
    
    Arguments
    ------------
    odb_id (odb object): odb file identifier
    stepnumber (int): step number for export, usually -1
    framenumber (list): frame number(s) for export; None gives all frames; 'skipfirst' gives all except 0
    
    Returns
    ------------
    sf_matrix: matrix with each frame as columns ( e.g. N_SF*N_MODES)
    label_vec: list with DOF labels of all N_DOF
    
    '''
    
    step_name=odb_id.steps.keys()[stepnumber]
    step_id=odb_id.steps[step_name]
    
    if framenumber is None:
        framenumber=range(len(step_id.frames))
    elif framenumber=='skipfirst':
        framenumber=range(1,len(step_id.frames))
    elif isinstance(framenumber,int):
        framenumber=np.array([framenumber])
    
    frame_id=step_id.frames[-1]
    if 'SF' not in frame_id.fieldOutputs.keys():
        sf_matrix=np.array([0 , 0 , 0])
        label_vec=np.array(['SF_not_found'])
        return sf_matrix,label_vec
    
    el_label_all=[ frame_id.fieldOutputs['SF'].values[n].baseElementType for n in range(len(frame_id.fieldOutputs['SF'].values)) ]
    index_B=[n for n, l in enumerate(el_label_all) if l.startswith('B')]
        
    sf_matrix=np.zeros((len(index_B)*6,len(framenumber)))
    
    for z in np.arange(len(framenumber)):
        
        frame_id=step_id.frames[framenumber[z]]
        
        output_sf=frame_id.fieldOutputs['SF']
        output_sf_val=output_sf.values
        
        output_sm=frame_id.fieldOutputs['SM']
        output_sm_val=output_sm.values
        
        sf_temp=np.array([output_sf_val[n].data for n in index_B ])
        sm_temp=np.array([output_sm_val[n].data for n in index_B ])
        
        output_sf_temp=np.hstack((sf_temp,sm_temp)).flatten()
        sf_matrix[:,z]=output_sf_temp
        
    sf_component_labels=output_sf.componentLabels
    
    # For SM:
    # Error in abaqus documentation? States 2 1 3 in odb file, but that is wrong.
    # >> OutputSM.componentLabels
    # >> ('SM2', 'SM1', 'SM3')    
    
    # Overwrite labels manually:    
    sm_component_labels=('SM1', 'SM2', 'SM3') 
    # NB! Important to verify results are reasonable
    
    label_vec_temp=[ [str(output_sf_val[n].elementLabel) + '_' + s  for s in sf_component_labels] + [str(output_sm_val[n].elementLabel) + '_' + s  for s in sm_component_labels] for n in index_B ]
    label_vec=[item for sublist in label_vec_temp for item in sublist]
    
    return sf_matrix,label_vec
    
    # From abaqus manual:
    # Section forces, moments, and transverse shear forces
    # SF1 Axial force.
    # SF2 Transverse shear force in the local 2-direction (not available for B23, B23H, B33, B33H).
    # SF3 Transverse shear force in the local 1-direction (available only for beams in space, not available for B33, B33H).
    # SM1 Bending moment about the local 1-axis.
    # SM2 Bending moment about the local 2-axis (available only for beams in space).
    # SM3 Twisting moment about the beam axis (available only for beams in space).

#%%

def exportelconn(odb_id):
    
    '''
    Export element connectivity (which elements are connected to which nodes)
    
    Arguments
    ------------
    odb_id (odb object): odb file identifier
    
    Returns
    ------------
    # elconn: matrix with rows [Elno,Nodeno_start,Nodeno_end]
    
    '''
    
    t_start=timer()

    instance_keys=odb_id.rootAssembly.instances.keys()
    
    for k in range(len(instance_keys)):
        
        instance_id=odb_id.rootAssembly.instances[instance_keys[k]]
        instance_elements=instance_id.elements
        
        eltype=[instance_elements[j].type for j in range(len(instance_elements))]
        
        idx_B31 = [i for i, s in enumerate(eltype) if 'B31' in s]
        idx_B33 = [i for i, s in enumerate(eltype) if 'B33' in s]
        idx_B32 = [i for i, s in enumerate(eltype) if 'B32' in s]
        
        elconn_B31=np.array( [ [instance_elements[j].label , int(instance_elements[j].connectivity[0]) , int(instance_elements[j].connectivity[1]) ]  for j in idx_B31 ] )
        elconn_B33=np.array( [ [instance_elements[j].label , int(instance_elements[j].connectivity[0]) , int(instance_elements[j].connectivity[1]) ]  for j in idx_B33 ] )
        elconn_B32=np.array( [ [instance_elements[j].label , int(instance_elements[j].connectivity[0]) , int(instance_elements[j].connectivity[1]) , int(instance_elements[j].connectivity[2]) ]  for j in idx_B32 ] )

    # If empty, then set shape to 3 or 4 columns
    if idx_B31==[]:
        elconn_B31=np.array([]).reshape(0,3)
    
    if idx_B33==[]:
        elconn_B33=np.array([]).reshape(0,3)
    
    if idx_B32==[]:
        elconn_B32=np.array([]).reshape(0,4)
        
    # Delete middle node for 3 node element
    elconn_B32_del=np.delete(elconn_B32,2,1)
    
    elconn=np.vstack((elconn_B31,elconn_B33,elconn_B32_del))
    
    t_end=timer()
    print('Time elconn ' + str(t_end-t_start))
    
    return elconn

#%%

def exportelsets(odb_id):
    
    '''
    Export element sets
    
    Arguments
    ------------
    odb_id (odb object): odb file identifier
    
    Returns
    ------------
    elset_numbers: vector with set number (separated by 0 between each set)
    elset_names: list with names
    
    '''
    
    instance_keys=odb_id.rootAssembly.instances.keys()
    elset_numbers=[]
    elset_names=[]
    
    t_start=timer()
    for k in range(len(instance_keys)):

        instance_id=odb_id.rootAssembly.instances[instance_keys[k]]
        elset_keys=instance_id.elementSets.keys()
        elset_names=np.append(elset_names,elset_keys)
        
        for i in range(len(elset_keys)):
            
            el_temp=instance_id.elementSets[elset_keys[i]].elements
            elnumbers_temp=[el_temp[ii].label for ii in range(len(el_temp))]
            elset_numbers=np.append(elset_numbers,np.append(elnumbers_temp,0))
            
    t_end=timer()
    print('Time elsets ' + str(t_end-t_start))
    return elset_numbers,elset_names
    
#%%

def save2txt(folder_save,name_save,A_matrix,atype='string',prefix=''):
    
    '''
    Export element sets
    
    Arguments
    ------------
    folder_save: string with folder name for export
    name_save: string with name for export
    A_matrix: the array to export
    atype: 'string' or 'number' specifies text or numeric data
    prefix: prefix in front of name_save
    
    Returns
    ------------
    None
    
    '''
    
    if atype=='number' or atype==1:
        np.savetxt((folder_save+'\\'+prefix+name_save+'.txt'),A_matrix ,delimiter=',',fmt='%.6e')
    elif atype=='string' or atype==2:
        np.savetxt((folder_save+'\\'+prefix+name_save+'.txt'),A_matrix ,delimiter=' ',fmt='%s')
    
#%%

def save2npy(folder_save,name_save,A_matrix,prefix=''):

    # Save numeric array or string array to npy file
    
    # Inputs:
    # folder_save: string with folder name for export
    # name_save: string with name for export
    # A_matrix: the array to export
    # prefix: prefix in front of name_save
    
    np.save((folder_save+'\\'+prefix+name_save),A_matrix)
    




import os
import sys
import numpy as np

folder_odb='C:/Cloud/OD_OWP/Work/Python/Github/abaqustools/odbexport_example'
jobname='Model_single_deck'
folder_save='C:/Cloud/OD_OWP/Work/Python/Github/abaqustools/odbexport_example'
folder_python='C:/Cloud/OD_OWP/Work/Python/Github/abaqustools/odbexport'
prefix='Model_single_deck_export_'

# Import functions for export (odbexport package)
sys.path.append(folder_python)
import odbfunc

# Open ODB
odb_id=odbfunc.open_odb(folder_odb,jobname)

# Step and frames to export
stepnumber=-1
framenumber='skipfirst'

# Node numbers to export
nodes=None

# Frequencies
f=odbfunc.exporthistoryoutput(odb_id,stepnumber,'EIGFREQ')
odbfunc.save2txt(folder_save,'f',f,atype=1,prefix=prefix)

# Generalized mass
gm=odbfunc.exporthistoryoutput(odb_id,stepnumber,'GM')
odbfunc.save2txt(folder_save,'gm',gm,atype=1,prefix=prefix)

# Mode shapes
(phi,phi_label)=odbfunc.exportdisplacement(odb_id,stepnumber,framenumber=framenumber,nodes=nodes)
odbfunc.save2txt(folder_save,'phi',phi,atype=1,prefix=prefix)
odbfunc.save2txt(folder_save,'phi_label',phi_label,atype=2,prefix=prefix)

# Modal section forces
(phi_sf,phi_sf_label)=odbfunc.exportsectionforce(odb_id,stepnumber,framenumber=framenumber)
odbfunc.save2txt(folder_save,'phi_sf',phi_sf,atype=1,prefix=prefix)
odbfunc.save2txt(folder_save,'phi_sf_label',phi_sf_label,atype=2,prefix=prefix)

# Node coordinates
nodecoord=odbfunc.exportnodecoord(odb_id,stepnumber,0)
odbfunc.save2txt(folder_save,'nodecoord',nodecoord,atype=1,prefix=prefix)

# Element connectivity
elconn=odbfunc.exportelconn(odb_id)
odbfunc.save2txt(folder_save,'elconn',elconn,atype=1,prefix=prefix)

# Close ODB
odbfunc.close_odb(odb_id)




# -*- coding: utf-8 -*-
"""
Created on 

"""
# Example truss model where diagonals are modelled with variable (used-defined)
# stiffness in joint connections

#%%

import numpy as np
import sys
sys.path.append('C:/Cloud/OD_OWP/Work/Python/Github')

from abaqustools import kw
from abaqustools import abq

#%%  Open input file

foldername=r'C:\Cloud\OD_OWP\Work\Python\Github\abaqustools\example\trussjointc'
input_filename='simple_trussjointc.inp'
jobname='simple_trussjointc'

fid=open(input_filename,'w')

#%%  Define structure

kw.material(fid,'DECK',15e3,0.3,1200)

kw.part(fid,'part_truss')

# Lower chord nodes
x1=np.array([0,6,10,14,20])
y1=x1*0
z1=x1*0
nodenumber1=np.arange(1,len(x1)+1)
node_matrix_bottom=np.column_stack((nodenumber1,x1,y1,z1))

# Upper chord nodes
x2=np.array([2,7.5,12.5,18])
y2=x2*0
z2=5*np.ones(np.shape(x2))
nodenumber2=np.arange(1,len(x2)+1)+10
node_matrix_top=np.column_stack((nodenumber2,x2,y2,z2))

node_matrix=np.vstack((node_matrix_bottom,node_matrix_top))
kw.node(fid,node_matrix,'NODES_CHORD')


#%%
# Elements and nodes for chord
nodes_el_chord=np.array([
    [100,1,2],
    [101,2,3],
    [102,3,4],
    [103,4,5],
    [201,11,12],
    [202,12,13],
    [203,13,14]
])

for k in np.arange(0,7):
    
    node1=nodes_el_chord[k,1]
    node2=nodes_el_chord[k,2]
    
    idx1=np.where(node_matrix[:,0]==node1)[0][0]
    coord1=node_matrix[idx1,1:]
    
    idx2=np.where(node_matrix[:,0]==node2)[0][0]
    coord2=node_matrix[idx2,1:]
    
    setname='Chord' + str(k+1)
    
    # Stiffness for x,y,z,rx,ry,rz (local system)
    # kj1=[1e12,1e12,1e12,1e12,1e12,1e12]
    kj1=[1e12,1e12,1e12,1e12,0,0]
    kj2=[1e12,1e12,1e12,1e12,0,0]
    
    # if k==0:
    #     kj1=[1e99,1e99,1e99,1e99,1e99,1e99]
    #     kj2=[1e12,1e12,1e12,1e12,0,0]
    # elif k==1:
    #     kj1=[1e12,1e12,1e12,1e12,0,0]
    #     kj2=[1e99,1e99,1e99,1e99,1e99,1e99]
    # else:
    #     kj1=[1e99,1e99,1e99,1e99,1e99,1e99]
    #     kj2=[1e99,1e99,1e99,1e99,1e99,1e99]
        
    
    if k==0:
        kj1=[1e99,1e99,1e99,1e99,1e99,1e99]
        #kj1=[1e12,1e12,1e12,1e12,0,0]
        kj2=[1e12,1e12,1e12,1e12,0,0]
    elif k==1:
        kj1=[1e12,1e12,1e12,1e12,0,0]
        kj2=[1e12,1e12,1e12,1e12,0,0]
    elif k==2:
        kj1=[1e12,1e12,1e12,1e12,0,0]
        kj2=[1e12,1e12,1e12,1e12,0,0]
    elif k==3:
       kj1=[1e12,1e12,1e12,1e12,0,0]
       kj2=[1e99,1e99,1e99,1e99,1e99,1e99]

    
    
    kw.elementjointc(fid, node1, node2, coord1 , coord2 , nodes_el_chord[k,0]*100, nodes_el_chord[k,0]*100,'B31',setname, [0,1,0],kj1,kj2,offset1=0.1,offset2=0.1,max_length=0.4) #

    kw.beamgeneralsection(fid, setname, 7850, [1e-3,1e-7,0,1e-6,1e-7], [0,1,0], [210e9,81e9])



#kw.element(fid,nodes_el_chord,'B31','Truss_chord')
#kw.beamgeneralsection(fid,'Truss_chord', 7850, [1e-4,1e-6,0,1e-6,1e-6], [0,1,0], [210e9,81e9])

#%%  Diagonals
nodes_el_dia=np.array([
    [301,1,11],
    [302,11,2],
    [303,2,12],
    [304,12,3],
    [305,3,13],
    [306,13,4],
    [307,4,14],
    [308,14,5]
])


for k in np.arange(0,8):
    
    node1=nodes_el_dia[k,1]
    node2=nodes_el_dia[k,2]
    
    idx1=np.where(node_matrix[:,0]==node1)[0][0]
    coord1=node_matrix[idx1,1:]
    
    idx2=np.where(node_matrix[:,0]==node2)[0][0]
    coord2=node_matrix[idx2,1:]
    
    setname='Dia' + str(k+1)
    
    # Stiffness for x,y,z,rx,ry,rz (local system)
    # kj1=[1e12,1e12,1e12,1e12,1e12,1e12]
    kj1=[1e10,1e10,1e10,1e10,0,0]
    kj2=[1e10,1e10,1e10,1e10,0,0]
    #kj2=[1e99,1e99,1e99,1e99,1e99,1e99]
       
    
    kw.elementjointc(fid, node1, node2, coord1 , coord2 , nodes_el_dia[k,0]*100,nodes_el_dia[k,0]*100,'B31',setname, [0,1,0],kj1,kj2,offset1=0.1,offset2=0.1,max_length=0.5) #

    kw.beamgeneralsection(fid, setname, 7850, [1e-3,1e-7,0,1e-6,1e-7], [0,1,0], [210e9,81e9])


kw.nset(fid,'support',[1,5])

#%%



kw.shellgrid(fid,'DECK','DECK','DECK',0.2,0,0,0,1,1,100,10,500e3,1e3)




#%% 

kw.partend(fid)

kw.comment(fid,'ASSEMBLY',True)
    
kw.assembly(fid,'as_truss')
    
kw.instance(fid,'part_truss','part_truss')
    
kw.instanceend(fid)
    
kw.assemblyend(fid)
    
#%%  Step static

# kw.step(fid,'NLGEO=NO, NAME=STEP_STATIC','Dead load')

# kw.static(fid,'1e-3, 1, 1e-6, 1')

# kw.gravload(fid,'new',[''],9.81)
    
# kw.boundary(fid,'new','support',[1,6,0],'part_truss')

# kw.boundary(fid,'new','NODES_BOTTOM',[2,2,0],'part_truss')
# kw.boundary(fid,'new','NODES_TOP',[2,2,0],'part_truss')


# kw.fieldoutput(fid,'NODE',['U' , 'RF' , 'COORD'],'','FREQUENCY=100')
# kw.fieldoutput(fid,'ELEMENT',['SF'],'','FREQUENCY=100')
    
# kw.stepend(fid)

#%%  Step modal analysis

kw.step(fid,'NAME=STEP_MODAL','')
kw.frequency(fid,50,'displacement')

kw.boundary(fid,'new','support',[1,4,0],'part_truss')

kw.boundary(fid,'new','NODES_CHORD',[2,2,0],'part_truss')
    
kw.fieldoutput(fid,'NODE',['U' , 'COORD'],'','')
kw.fieldoutput(fid,'ELEMENT',['SF','S'],'','')

kw.stepend(fid)

#%%  Close file

fid.close()

#%%  Run job
    
# Check input file for duplicate node or element numbers
abq.checkduplicate(input_filename)
    
abq.runjob(foldername,input_filename,jobname)



